<?php

namespace Otys\OtysPlugin\Controllers;

use Otys\OtysPlugin\Includes\OtysApi\OtysApi;
use Otys\OtysPlugin\Includes\Core\Routes;
use Otys\OtysPlugin\Includes\Core\Recaptcha;
use Otys\OtysPlugin\Includes\Core\Hooks;

use \DateTime as DateTime;

final class VacanciesController extends BaseController
{
    public function __construct()
    {
        parent::__construct();
        $this->model->modulename = 'vacancies';
    }

    /**
     * Callback shortList shortcode
     *
     * @param array $atts
     * @param array $args
     * @return void
     * @since 1.0.0
     */
    public function shortList($atts = [], $args = []): void
    {
        $otysApi = new OtysApi;

        $atts = wp_parse_args($atts, [
            'limit' => 3
        ]);

        if (isset($atts['limit'])) {
            $args['limit'] = intval($atts['limit']);
        }

        // get match criteria
        $matchCriteriaLabels = get_option("otys_option_vacancies_list_match_criteria_labels", []);
  
        // Get match criteria labels
        if (is_array($matchCriteriaLabels)) {
            foreach ($matchCriteriaLabels as $criteriaLabel) {
                $criteriaId = str_replace("C_", "", $criteriaLabel);
                $args["what"]["matchCriteria_{$criteriaId}"] = true;
            }
        }

        /**
         * Apply selected match criteria
         */
        $args['selectedMatchcriteria'] = [];

        $criteriaSlugs = $this->model->getMatchCriteriaSlugs();

        if (is_array($atts)) {
            foreach ($atts as $criteria => $value) {
                // Add criteria from attributes based on id's
                if (strpos($criteria, 'criteria-') !== false) {
                    $criteriaId = str_replace("criteria-", "C_", $criteria);
                    $args['selectedMatchcriteria'][$criteriaId] = explode(',', $value);
                }

                // Add criteria from attributes based on slugs
                if (array_key_exists($criteria, $criteriaSlugs)) {
                    $optionsSlugged = explode(',', $value);

                    foreach ($optionsSlugged as $optionSlugged) {
                        if (array_key_exists($optionSlugged, $criteriaSlugs[$criteria]['options'])) {
                            $args['selectedMatchcriteria'][$criteriaSlugs[$criteria]['id']][] = $criteriaSlugs[$criteria]['options'][$optionSlugged]['id'];
                        }
                    }
                }
            }
        }

        // Get the current selected match criteria
        $selectedMatchCriteria = $this->model->getSelectedMatchCriteria($args['selectedMatchcriteria']);

        /* Add selected match criteria to the criteria list of the vacancies arguments so
        the result will be based on the selected criteria */
        $args["search"]["ACTONOMY"]["DATA"]["matchcriteria"] = $selectedMatchCriteria;

        // Get vacancies list
        $vacancies = $otysApi->vacancies->getList(99, 1, $args);

        $vacanciesList = ($vacancies !== false && array_key_exists('listOutput', $vacancies)) ? $vacancies['listOutput'] : [];

        if (!empty($vacanciesList)) {
            /**
             * Add apply url's
             */
            foreach ($vacanciesList as $key => $vacancy) {
                $vacanciesList[$key]['url'] = Routes::get('vacancy-detail', [
                    'name' => sanitize_title($vacancy['title']) . '-' . $vacancy['uid']
                ]);
            }
        }

        $totalVacancies = !empty($vacancies) ? $vacancies['searchExtras']['ACTONOMY']['totalCount'] : 0;

        $this->parseArgs('vacancies', $vacanciesList);
        $this->parseArgs('matchCriteriaList', $this->model->getMatchCriteriaList());
        $this->parseArgs('vacanciesTotal', $totalVacancies);
        $this->setTemplate('vacancies/vacancies-shortlist.php');
    }

    /**
     * Display list of vacancies
     *
     * @param array $atts
     * @return void
     * @since 1.0.0
     */
    public function list($atts = [], $args = [])
    {
        $otysApi = new OtysApi;
        $pagination = [];

        $args = wp_parse_args($args, [
            'perPage' => get_option("otys_option_vacancies_per_page", 10),
            'currentPage' => 1,
            'selectedMatchcriteria' => []
        ]);

        // Check if we want to search on keywords
        if (isset($_GET['search']) && !empty($_GET['search'])) {
            $search = (string) (is_string($_GET['search']) ? preg_replace('/[^a-zA-Z0-9 .,&\*\']/', '', $_GET['search']) : '');
        
            if ($search !== '') {
                // Add keyword search to our vacancies args
                $args["search"]["ACTONOMY"]["DATA"]["keywords"] = [
                    "options" => [
                        "enableExpansion" => false,
                        "required" => true,
                        "searchList" => [
                            "chapo",
                            "description",
                            "profile_culture",
                            "salary",
                            "title"
                        ],
                        "searchMode" => "all"
                    ],
                    "value" => "{$search}"
                ];
            }
        }

        // Set the amount of vacancies per page 
        if (isset($atts['perpage'])) {
            $args['perPage'] = $atts['perpage'];
        }

        /**
         * Current page and offset
         */
        $pageNumber = isset($_GET['page-number']) ? sanitize_key($_GET['page-number']) : 1;
        if (isset($pageNumber) && is_numeric($pageNumber)) {
            $args["currentPage"] = intval($pageNumber);
        } else {
            $args["currentPage"] = 1;
        }

        /**
         * Apply selected match criteria
         */
        $args['selectedMatchcriteria'] = [];

        $criteriaSlugs = $this->model->getMatchCriteriaSlugs();

        if (is_array($atts)) {
            foreach ($atts as $criteria => $value) {
                // Add criteria from attributes based on id's
                if (strpos($criteria, 'criteria-') !== false) {
                    $criteriaId = str_replace("criteria-", "C_", $criteria);
                    $args['selectedMatchcriteria'][$criteriaId] = explode(',', $value);
                }

                // Add criteria from attributes based on slugs
                if (array_key_exists($criteria, $criteriaSlugs)) {
                    $optionsSlugged = explode(',', $value);

                    foreach ($optionsSlugged as $optionSlugged) {
                        if (array_key_exists($optionSlugged, $criteriaSlugs[$criteria]['options'])) {
                            $args['selectedMatchcriteria'][$criteriaSlugs[$criteria]['id']][] = $criteriaSlugs[$criteria]['options'][$optionSlugged]['id'];
                        }
                    }
                }
            }
        }

        // Get the current selected match criteria
        $selectedMatchCriteria = $this->model->getSelectedMatchCriteria($args['selectedMatchcriteria']);

        /* Add selected match criteria to the criteria list of the vacancies arguments so
        the result will be based on the selected criteria */
        $args["search"]["ACTONOMY"]["DATA"]["matchcriteria"] = $selectedMatchCriteria;

        /**
         * Match criteria labels
         */
        $matchCriteriaLabels = get_option("otys_option_vacancies_list_match_criteria_labels", []);

        /**
         * Get match criteria labels
         */
        if (is_array($matchCriteriaLabels)) {
            foreach ($matchCriteriaLabels as $criteriaLabel) {
                $criteriaId = str_replace("C_", "", $criteriaLabel);
                $args["what"]["matchCriteria_{$criteriaId}"] = true;
            }
        }

        $vacancies = $otysApi->vacancies->getList(intval($args['perPage']), intval($args["currentPage"]), $args);
        $totalVacancies = !empty($vacancies) ? $vacancies['searchExtras']['ACTONOMY']['totalCount'] : 0;

        if (!empty($vacancies)) {

            /**
             * Add apply url's
             */
            foreach ($vacancies['listOutput'] as $key => $vacancy) {
                $vacancies['listOutput'][$key]['url'] = Routes::get('vacancy-detail', [
                    'name' => sanitize_title($vacancy['title']) . '-' . $vacancy['uid']
                ]);
            }

            /**
             * Pagination
             */
            
            // Set current page
            $currentPageNr = (isset($_GET['page-number']) && is_numeric($_GET['page-number'])) ? sanitize_key($_GET['page-number']) : 1;

            // Get pagination
            $pagination = $this->model->getPagination(
                intval($args['perPage']),
                intval($totalVacancies),
                intval($currentPageNr),
                array_keys($this->model->getMatchCriteriaSlugs())
            );

        }
        
        $vacancyList = ($vacancies !== false && array_key_exists('listOutput', $vacancies)) ? $vacancies['listOutput'] : [];
        
        $this->parseArgs('pagination', $pagination);
        $this->parseArgs('vacancies', $vacancyList);
        $this->parseArgs('vacanciesTotal', $totalVacancies);
        $this->parseArgs('query_parameters', $this->model->getSelectedCriteria($args['selectedMatchcriteria']));

        $this->setTemplate('vacancies/vacancies-list.php');
    }

    /**
     * Vacancy filters shortcode callback
     *
     * @param array $selectedCriteria
     * @return void
     * @since 1.0.0
     */
    public function filters(array $selectedCriteria): void
    {
        $this->parseArgs('matchCriteriaList', $this->model->getMatchCriteriaList($selectedCriteria));

        $this->setTemplate('vacancies/vacancies-filters.php');
    }

    /**
     * Vacancy Detail
     *
     * @return void
     * @since  1.0.0
     */
    public function detail(): void
    {
        global $post;

        // Init vars
        $otysApi = new OtysApi();
        $website = intval(get_option('otys_option_website'));
        $today = new DateTime();

        $vacancyUid = $otysApi->vacancies->getVacancyUidFromUrl();
            
        // Get the match criteria labels to show on the vacancy detail
        $matchCriteriaLabels = get_option('otys_option_vacancies_detail_match_criteria_labels', []);

        $vacanciesArgs = [];

        // Add the match criteria labels to show to the ows request
        if (is_array($matchCriteriaLabels)) {
            foreach ($matchCriteriaLabels as $criteriaLabel) {
                $criteriaId = str_replace('C_', '', $criteriaLabel);
                $vacanciesArgs["matchCriteria_{$criteriaId}"] = true;
            }
        }

        // Get vacancy
        $vacancy = $otysApi->vacancies->get($vacancyUid, $vacanciesArgs);

        // Check publication date
        $publicationStartDate = new DateTime($vacancy['publicationStartDate']);
        $publicationEndDate = new DateTime($vacancy['publicationEndDate'] . "23:59:59");

        if ($publicationStartDate > $today || $publicationEndDate < $today) {
            Routes::throwError(404);
        }

        // Check if vacancy is found is published for the current website
        if (empty($vacancy) || !$vacancy["published"] || ($website && !array_key_exists($website, $vacancy["publishedWebsites"]))) {
            Routes::throwError(404);
        }

        // Add apply url
        $vacancy['applyUrl'] = Routes::get('vacancy-apply', [
            'name' => sanitize_title($vacancy['title']) . '-' . $vacancy['uid']
        ]);

        $post->post_title = sprintf(
            __('Vacancy %1$s', 'otys-jobs-apply'),
            $vacancy['title']
        );

        $this->parseArgs('vacancy', $vacancy);

        $this->setTemplate('vacancies/vacancies-detail.php');
    }

    /**
     * Callback function for Vacancy Detail
     *
     * @return void
     * @since 1.0.0
     */
    public static function detailCallback(): void
    {
        /**
         * When wp_head action gets fired when want to implement Google for jobs
         * within the head tags. Therefore we add a callback action.
         */
        Hooks::addAction('wp_head', static::class, 'googleForJobs', 10, 1);
    }

    /**
     * Google for jobs header callback
     * Function is called when wp_head action is triggered
     *
     * @param mixed $headers
     * @return mixed
     * @since 1.0.0
     */
    public static function googleForJobs($headers)
    {
        $otysApi = new OtysApi();

        $vacancyUid = $otysApi->vacancies->getVacancyUidFromUrl();
        $googleForJobs = $otysApi->vacancies->getGoogleForJobs($vacancyUid);

        $googleForJobsObject = [
            "@context" => "https://schema.org/",
            "@type" => "JobPosting",
            "identifier" => [
                "@type" => "PropertyValue"
            ],
            "hiringOrganization" => [
                "@type" => "Organization"
            ],
            "jobLocation" => [
                "@type" => "Place",
                "address" => [
                    "@type" => "PostalAddress"
                ]
            ],
            "baseSalary" => [
                "@type" => "MonetaryAmount",
                "value" => [
                    "@type" => "QuantitativeValue"
                ]
            ]
        ];

        foreach ($googleForJobs as $name => $value) {
            $explode = explode('___', $name);
            $array = $otysApi->vacancies->flatArrayToMultidimensional($explode, $value);

            $googleForJobsObject = array_merge_recursive($googleForJobsObject, $array);
        }

        static::loadTemplate('/vacancies/google-for-jobs.php', $googleForJobsObject);

        return $headers;
    }

    /**
     * Vacancy application page
     *
     * @return void
     * @since 1.0.0
     */
    public function apply(): void
    {
        global $wp;

        // Init vars
        $otysApi = new OtysApi();
        $website = intval(get_option('otys_option_website')); // Current website ID
        $vacancyUid = $otysApi->vacancies->getVacancyUidFromUrl(1); // Vacancy UID from url
        $vacancy = $otysApi->vacancies->get($vacancyUid);
        $errors = [];

        // Check if vacancy is found is published for the current website
        if (empty($vacancy) || !$vacancy["published"] || ($website && !array_key_exists($website, $vacancy["publishedWebsites"]))) {
            Routes::throwError(404);
        }
        
        // Get the questionset ID
        $questionsetId = $otysApi->candidateQuestionSets->getVacancyQuestionsetId($vacancyUid);

        if (empty($questionset = $otysApi->candidateQuestionSets->getQuestionSet($questionsetId))) {
            // If the questionset has not been found, throw a 404 page
            Routes::throwError(404);
        }

        // Init vars when vacancy and questionset is confirmed to exist
        $postedData = $otysApi->candidateQuestionSets->getData($vacancyUid); // Previously posted filled in data stored in Session
        $totalPages = count($questionset['pages']); // Total pages in the questionset
        $currentPage = $questionset['currentPage'];

        // If the current page does not exist, redirect to a existing page
        if ($currentPage > $totalPages) {
           
            // If the current page is higher than the last page redirect to the last page
            $previousPageUrls = add_query_arg(['page-number' => $totalPages], home_url($wp->request));
            
            wp_redirect($previousPageUrls);
            exit;
        } else if ($currentPage < 1) {
          
            // If the current page is lower than 1 redirect to page one
            $previousPageUrls = remove_query_arg('page-number', home_url($wp->request));

            wp_redirect($previousPageUrls);
            exit;
        }

        /**
         * Handle POST
         */
        if (isset($_POST) && isset($_POST['uid'])) {            
            $_POST = filter_input_array(INPUT_POST, FILTER_SANITIZE_FULL_SPECIAL_CHARS);

            // Save POST data in session so we can use it in other pages
            $otysApi->candidateQuestionSets->savePost($vacancyUid, $questionsetId, $_POST, $currentPage);

            /**
             * Handle file upload
             */
            if (isset($_FILES) && !empty($_FILES) && isset($_FILES)) {
                /**
                 * We'll loop through all file uploads and handle the files
                 */
                foreach ($_FILES as $questionId => $upload) {
                    if (!array_key_exists($questionId, $questionset['pages'][$currentPage]['questions'])) {
                        // Check if FILE upload is within questionset if not skip this file upload
                        continue;
                    }

                    // Get current question and question type from the questionset
                    $question = $questionset['pages'][$currentPage]['questions'][$questionId];
                    $questionType = $question['type'];

                    // If the question set type is not type file skip this file
                    if ($questionType !== 'multifile' && $questionType !== 'file') {
                        continue;
                    }

                    $currentQuestionAnswer = (isset($postedData['answers'][$currentPage][$questionId]['documents']) ? $postedData['answers'][$currentPage][$questionId]['documents'] : []);
                    $required = $question['validation']['mandatory'];

                    // If type is multifile we want to add on the current data else we start blank
                    $documentsData = $questionType === 'multifile' ? $currentQuestionAnswer : [];

                    // Handle documents via OWS
                    $documents = $otysApi->documents->addDocuments($upload);

                    // Loop through the document response and handle possible errors and save post data
                    foreach ($documents as $document) {
                        if (is_wp_error($document["response"])) {
                            // If an error occursed while uploading 
                            if ((!$required || !empty($currentQuestionAnswer)) && $document['file']['error'] === 4) {
                            } else {
                                // Add error and go further with the next document
                                $errors[$questionId]['document'] = $document['response']->get_error_message();
                            }
                        } else {
                            $documentsData[] = $document['response'];
                        }
                    }

                    // Remember data for next page
                    $otysApi->candidateQuestionSets->addData($vacancyUid, $currentPage, [$questionId => ['documents' => $documentsData]]);
                }
            }

            /**
             * Verify Recaptcha
             */
            if ($currentPage === $totalPages) {
                $recaptcha = new Recaptcha();

                if (!isset($_POST['g-recaptcha-response']) || !$recaptcha->verify(filter_var($_POST['g-recaptcha-response'], FILTER_SANITIZE_SPECIAL_CHARS))) {
                    $errors['recaptcha'] = __('reCAPTCHA is incorrect. Please try again.', 'otys-jobs-apply');
                }
            }

            /**
             * Error check
             */
            $validate = $otysApi->candidateQuestionSets->validate(
                $questionset['pages'][$currentPage]['questions'],
                $otysApi->candidateQuestionSets->getPageData($vacancyUid, $currentPage)
            );

            /**
             * Assign errors
             */
            $errors = array_replace_recursive($errors, $validate['errors']);

            /**
             * Handle form if there are no errors
             */
            if (empty($errors)) {
                $fieldValues = $otysApi->candidateQuestionSets->getFieldValues($vacancyUid, $questionsetId);

                if ($currentPage === $totalPages) {
                    $currentUrl = home_url($wp->request);
                    
                    // Get document uid which will be used for sending the confirm email
                    $documentUid = intval(get_option('otys_option_document_template_apply_confirm_email', 0));

                    /**
                     * If the documentUid equals 0 the confirmation email will not be send and the candidate will be 
                     * added anyway. If there is a document ID and the candidate already exist we will send a confirmation
                     * email to the candidate with a code which the candidate needs to fill in on the next page
                     */
                    if ($documentUid !== 0 && !empty($candidate = $otysApi->candidates->getCandidate($fieldValues['Person.emailPrimary']))) {

                        // Generate confirmation code
                        $confirmationCode = rand(10000,99999);

                        // Get the mail layout for the the email confirmation
                        $document = $otysApi->documents->getDocument($documentUid, [
                            'relatedEntities' =>  [
                                [
                                    "entityId" => 1, /* vacancy */
                                    "recordId" => "{$vacancyUid}"
                                ],
                                [
                                    "entityId" => 2, /* candidate */
                                    "recordId" => $candidate['uid']
                                ],
                            ],
                            "answers" => [
                                "code" => "{$confirmationCode}"
                            ]
                        ]);

                        // If we where able to retrieve a document send the document as email
                        if (!empty($document)) {
                            // Save the code in the session so we can use the code on other pages
                            $otysApi->candidateQuestionSets->setData($vacancyUid, [
                                "code" => "{$confirmationCode}"
                            ]);
                
                            // Send the mail with the code to the user
                            $otysApi->mail->send([
                                "subject" => $document['subject'],
                                "htmlMessage" => $document['htmlBody'],
                                "To" => [
                                    $fieldValues['Person.emailPrimary']
                                ]
                            ]);
                        } else {
                            /**
                             * Fallback for when retrieving the document failed. When the document
                             * response failed we will just a continue the application
                             */
                            if ($this->doApplication($vacancyUid)) {
                                $thankYouPage = Routes::get('vacancy-apply-success', [
                                    'name' => sanitize_title($vacancy['title']) . '-' . $vacancy['uid']
                                ]);

                                $otysApi->candidateQuestionSets->removeData($questionsetId);

                                wp_redirect(esc_url_raw($thankYouPage));
                                exit;
                            } else {
                                echo __('Something went wrong, please contact ths server admin', 'otys-jobs-apply');
                            }
                        }

                        /**
                         * If email already exist we need the user to confirm the email adres
                         */
                        $redirectUrl = Routes::get('vacancy-apply-confirm-email', [
                            'name' => sanitize_title($vacancy['title']) . '-' . $vacancy['uid']
                        ]);

                        wp_redirect(esc_url_raw($redirectUrl));
                        exit;
                    } else {
                        /**
                         * Handle the application
                         */
                        if ($this->doApplication($vacancyUid)) {
                            $thankYouPage = Routes::get('vacancy-apply-success', [
                                'name' => sanitize_title($vacancy['title']) . '-' . $vacancy['uid']
                            ]);

                            wp_redirect(esc_url_raw($thankYouPage));
                            exit;
                        } else {
                            echo __('Something went wrong, please contact ths server admin', 'otys-jobs-apply');
                        }
                    }
                } else {
                    /**
                     * Send the user to the next page
                     */
                    $nextPageNumber = $currentPage + 1;
                    $nextPageUrl = add_query_arg(['page-number' => $nextPageNumber], home_url($wp->request));

                    wp_redirect($nextPageUrl);
                    exit;
                }
            }
        }

        /**
         * Check if errors should be displayed initially without posting
         */
        if (isset($_GET['displayerrors']) && $_GET['displayerrors']) {
            // Validate errors and get potential errors from the current POST
            $validate = $otysApi->candidateQuestionSets->validate(
                $questionset['pages'][$currentPage]['questions'],
                $otysApi->candidateQuestionSets->getPageData($vacancyUid, $currentPage)
            );

            $errors = $validate['errors'];
        }

        $postedData = $otysApi->candidateQuestionSets->getData($vacancyUid);

        $otysApi->candidateQuestionSets->resetErrors($questionsetId);

        $ajaxUrl = Routes::get('ajax');

        $this->setTemplate('vacancies/vacancies-apply.php');
        
        $this->parseArgs('ajaxUrl', $ajaxUrl);
        $this->parseArgs('formData', !empty($postedData) && isset($postedData['answers'][$currentPage]) ? $postedData['answers'][$currentPage] : []);
        $this->parseArgs('vacancy', $vacancy);
        $this->parseArgs('questionset', $questionset);
        $this->parseArgs('errors', $errors);
    }

    /**
     * Callback function for Application page
     *
     * @param mixed $atts Attributes supplied to the shortcode
     * @return void
     * @since 1.0.0
     */
    public function applyShortcode($atts = '')
    {
        global $wp;

        $atts = wp_parse_args($atts, [
            'redirect' => ''
        ]);

        // Init vars
        $otysApi = new OtysApi();
        $website = intval(get_option('otys_option_website')); // Current website ID
        $errors = [];


        if(empty($questionsetData = $otysApi->candidateQuestionSets->getOpenQuestionset())) {
            return;
        }

        // Get the questionset ID
        $questionsetId = sanitize_key($questionsetData['uid']);

        // Retrieve questionset
        if (empty($questionset = $otysApi->candidateQuestionSets->getQuestionSet($questionsetId))) {
            return;
        }

        // Init vars when vacancy and questionset is confirmed to exist
        $postedData = $otysApi->candidateQuestionSets->getData($questionsetId); // Previously posted filled in data stored in Session
        $totalPages = count($questionset['pages']); // Total pages in the questionset
        $currentPage = $questionset['currentPage'];

        if (isset($postedData['errors'])) {
            $errors = $postedData['errors'];
        }

        // If the current page does not exist, redirect to a existing page
        if ($currentPage > $totalPages) {
            // If the current page is higher than the last page redirect to the last page
            $previousPageUrls = add_query_arg(['page-number' => $totalPages], home_url($wp->request));

            wp_redirect($previousPageUrls);
            exit;
        } else if ($currentPage < 1) {
            // If the current page is lower than 1 redirect to page one
            $previousPageUrls = remove_query_arg('page-number', home_url($wp->request));

            wp_redirect($previousPageUrls);
            exit;
        }

        /**
         * Check if errors should be displayed initially without posting
         */
        if (isset($_GET['displayerrors']) && $_GET['displayerrors']) {
            // Validate errors and get potential errors from the current POST
            $validate = $otysApi->candidateQuestionSets->validate(
                $questionset['pages'][$currentPage]['questions'],
                $otysApi->candidateQuestionSets->getPageData($questionsetId, $currentPage)
            );

            $errors = $validate['errors'];
        }

        $otysApi->candidateQuestionSets->resetErrors($questionsetId);

        $this->setTemplate('vacancies/vacancies-open-apply.php');

        if (isset($postedData['success']) && $postedData['success'] === true) {
            $this->setTemplate('vacancies/vacancies-open-apply-success.php');

            $otysApi->candidateQuestionSets->removeData($questionsetId);
        }
        
        $this->parseArgs('ajaxUrl', Routes::get('ajax'));
        $this->parseArgs('redirect', $atts['redirect']);
        $this->parseArgs('formData', !empty($postedData) && isset($postedData['answers'][$currentPage]) ? $postedData['answers'][$currentPage] : []);
        $this->parseArgs('questionset', $questionset);
        $this->parseArgs('errors', $errors);
    }

    /**
     * Undocumented function
     * @return mixed
     * @since 1.0.0
     */
    public static function processApplyShortcode()
    {
        /**
         * Handle POST
         */
        if (isset($_POST) && isset($_POST['action']) && $_POST['action'] === 'openapplication') {
            global $wp;

            // Init vars
            $otysApi = new OtysApi();
            $website = intval(get_option('otys_option_website')); // Current website ID
            $redirect = isset($_POST['redirect']) ?  sanitize_url(get_permalink(get_page_by_path(sanitize_title($_POST['redirect'])))) : '';
            $errors = [];
            $_POST = filter_input_array(INPUT_POST, FILTER_SANITIZE_FULL_SPECIAL_CHARS);
            
            if(empty($questionset = $otysApi->candidateQuestionSets->getOpenQuestionset())) {
                return;
            }

            // Get the questionset ID
            $questionsetId = sanitize_key($questionset['uid']);
                
            $otysApi->candidateQuestionSets->removeData($questionsetId, 'errors');

            // Retrieve questionset
            if (empty($questionset = $otysApi->candidateQuestionSets->getQuestionSet($questionsetId))) {
                return;
            }

            // Init vars when vacancy and questionset is confirmed to exist
            $postedData = $otysApi->candidateQuestionSets->getData($questionsetId); // Previously posted filled in data stored in Session
            $totalPages = count($questionset['pages']); // Total pages in the questionset
            $currentPage = $questionset['currentPage'];

            // If the current page does not exist, redirect to a existing page
            if ($currentPage > $totalPages) {
                // If the current page is higher than the last page redirect to the last page
                $previousPageUrls = add_query_arg(['page-number' => $totalPages], home_url($wp->request));

                wp_redirect($previousPageUrls);
                exit;
            } else if ($currentPage < 1) {
                // If the current page is lower than 1 redirect to page one
                $previousPageUrls = remove_query_arg('page-number', home_url($wp->request));

                wp_redirect($previousPageUrls);
                exit;
            }
            

            // Filter input $_POST
            array_filter($_POST, function(&$value) {
                $value = trim($value);
            });

            $_POST = filter_input_array(INPUT_POST, FILTER_SANITIZE_FULL_SPECIAL_CHARS);

            // Save POST data in session so we can use it in other pages
            $otysApi->candidateQuestionSets->savePost($questionsetId, $questionsetId, $_POST, $currentPage);

            /**
             * Handle file upload
             */
            if (isset($_FILES) && !empty($_FILES) && isset($_FILES)) {
                /**
                 * We'll loop through all file uploads and handle the files
                 */
                foreach ($_FILES as $questionId => $upload) {
                    if (!array_key_exists($questionId, $questionset['pages'][$currentPage]['questions'])) {
                        // Check if FILE upload is within questionset if not skip this file upload
                        continue;
                    }

                    // Get current question and question type from the questionset
                    $question = $questionset['pages'][$currentPage]['questions'][$questionId];
                    $questionType = $question['type'];

                    // If the question set type is not type file skip this file
                    if ($questionType !== 'multifile' && $questionType !== 'file') {
                        continue;
                    }

                    $currentQuestionAnswer = (isset($postedData['answers'][$currentPage][$questionId]['documents']) ? $postedData['answers'][$currentPage][$questionId]['documents'] : []);
                    $required = $question['validation']['mandatory'];

                    // If type is multifile we want to add on the current data else we start blank
                    $documentsData = $questionType === 'multifile' ? $currentQuestionAnswer : [];

                    // Handle documents via OWS
                    $documents = $otysApi->documents->addDocuments($upload);

                    // Loop through the document response and handle possible errors and save post data
                    foreach ($documents as $document) {
                        if (is_wp_error($document["response"])) {
                            // If an error occured while uploading 
                            if ((!$required || !empty($currentQuestionAnswer)) && $document['file']['error'] === 4) {
                            } else {
                                // Add error and go further with the next document
                                $errors[$questionId]['document'] = $document['response']->get_error_message();
                            }
                        } else {
                            $documentsData[] = $document['response'];
                        }
                    }

                    // Remember data for next page
                    $otysApi->candidateQuestionSets->addData($questionsetId, $currentPage, [$questionId => ['documents' => $documentsData]]);
                }
            }

            /**
             * Verify Recaptcha
             */
            if ($currentPage === $totalPages) {
                $recaptcha = new Recaptcha();

                if (!isset($_POST['g-recaptcha-response']) || !$recaptcha->verify(filter_var($_POST['g-recaptcha-response'], FILTER_SANITIZE_SPECIAL_CHARS))) {
                    $errors['recaptcha'] = __('reCAPTCHA is incorrect. Please try again.', 'otys-jobs-apply');
                }
            }

            /**
             * Error check
             */
            $validate = $otysApi->candidateQuestionSets->validate(
                $questionset['pages'][$currentPage]['questions'],
                $otysApi->candidateQuestionSets->getPageData($questionsetId, $currentPage),
                true,
                [
                    'uniqueEmail' => true
                ]
            );

            $fieldValues = $otysApi->candidateQuestionSets->getFieldValues($questionsetId, $questionsetId);

            /**
             * Assign errors
             */
            $errors = array_replace_recursive($errors, $validate['errors']);

            /**
             * Save errors in session for next page
             */
            $otysApi->candidateQuestionSets->setData($questionsetId, [
                'errors' => $errors
            ]);

            /**
             * Handle form if there are no errors
             */
            if (empty($errors)) {
                if ($currentPage === $totalPages) {
                    /**
                     * Create candidate
                     */
                    $candidateUid = $otysApi->candidates->createCandidate($fieldValues);

                    if(!empty($candidateUid)) {
                        /**
                         * Loop through all postdata to check if there are files which need to be assigned to the current candidate
                          */ 
                        foreach ($fieldValues as $fieldName => $postQuestionAnswer) {
                            // Check if current answer has documents
                            if (isset($postQuestionAnswer['documents'])) {
                                // Loop through all documents and attach them to the current applying candidate
                                foreach ($postQuestionAnswer['documents'] as $file) {
                                    $documentResult = $otysApi->documents->attachDocument([
                                        "subject" => $file['name'],
                                        "typeId" => ($fieldName == 'cv') ? "1" : "201", // type 1 = CV, type 201 = Other
                                        "private" => "false",
                                        "customerRightsLevel" => 1,
                                        "alwaysOnTop" => "false",
                                        "candidateUid" => $candidateUid,
                                        "fileUid" => $file['ouid']
                                    ]);
                                }
                            }
                        }

                        // $otysApi->candidateQuestionSets->removeData($questionsetId, 'answers');
                        $otysApi->candidateQuestionSets->setData($questionsetId, ['success' => true]);

                        $webuser = $otysApi->candidates->createWebuser($candidateUid, $fieldValues['Person.emailPrimary']);

                        $webuserPassword = isset($webuser['password']) ? $webuser['password'] : '';

                        /**
                         * Notify candidate by email that the application was succesfull
                         */
                        $documentUid = intval(get_option('otys_option_document_template_open_apply_notify_candidate', 0));

                        // If documentuid equals 0 we won't send the candidate a notification email
                        if ($documentUid !== 0) {
                            // Get the document used for the candidate notification email
                            $document = $otysApi->documents->getDocument($documentUid, [
                                'relatedEntities' =>  [
                                    [
                                        "entityId" => 2, /* candidate */
                                        "recordId" => "{$candidateUid}"
                                    ]
                                ],
                                "answers" => [
                                    "password" => "{$webuserPassword}"
                                ]
                            ]);

                            if (!empty($document)) {
                                // Send the notifcation email to the candidate
                                $otysApi->mail->send([
                                    "subject" => $document['subject'],
                                    "htmlMessage" => $document['htmlBody'],
                                    "To" => [
                                        $fieldValues['Person.emailPrimary']
                                    ]
                                ]);
                            }
                        }

                        /**
                         * Redirect to the thank you page if a thank you page issset
                         */
                        if ($redirect !== '') {
                            $otysApi->candidateQuestionSets->removeData($questionsetId);
                            wp_redirect($redirect);
                            exit;
                        }
                    }
                } else {
                    /**
                     * Send the user to the next page
                     */
                    $nextPageNumber = $currentPage + 1;
                    $nextPageUrl = add_query_arg(['page-number' => $nextPageNumber], home_url($wp->request));
    
                    wp_redirect($nextPageUrl);
                }
            }
        }
    }

    /**
     * Do application for vacancy
     *
     * @param string $vacancyUid
     * @return boolean
     * @since 1.0.0
     */
    private function doApplication(string $vacancyUid): bool
    {
        $otysApi = new OtysApi;

        $vacancy = $otysApi->vacancies->get($vacancyUid);

        $questionsetId = ($vacancy['questionSetId'] === 0) ? $otysApi->candidateQuestionSets->getVacancyQuestionsetId($vacancyUid) : $vacancy['questionSetId'];

        // Get all post data based on the questionset used
        $fieldValues = $otysApi->candidateQuestionSets->getFieldValues($vacancyUid, $questionsetId);
        $candidate = $otysApi->candidates->getCandidate($fieldValues['Person.emailPrimary']);

        // If a candidate with the given email exists, use that candidate to go further (so don't create a new candidate)
        /**
         * Translate field values keys to OWS fields
         */
        $candidateData = [];

        /**
         * Assign utm tags to the candidate data
         */
        $candidateData['UtmTags'] = filter_var($_SESSION['utm_tags'], FILTER_CALLBACK, array('options' => [$this, 'sanitizeUTM']));

        $candidateUid = (!empty($candidate)) ? $candidate['uid'] : $otysApi->candidates->createCandidate($fieldValues);

        // TODO build in fallback when createCandidate failed
        
        // If candidate exists update the candidate with the new filled in information
        if (!empty($candidate)) {
            $otysApi->candidates->updateCandidate($candidateUid, $fieldValues);
        }

        // Get procedure statusses to set
        $procedurestatusList = $otysApi->procedures->getStatusAutoSelect(1);

        $procedureArgs = [];

        if (isset($procedurestatusList[1])) {
            $procedureArgs['ProcedureStatus1']['uid'] = "1_{$procedurestatusList[1]['uid']}";
        }

        if (isset($procedurestatusList[2])) {
            $procedureArgs['ProcedureStatus2']['uid'] = "2_{$procedurestatusList[2]['uid']}";
        }

        if (isset($procedurestatusList[3])) {
            $procedureArgs['ProcedureStatus3']['uid'] = "3_{$procedurestatusList[3]['uid']}";
        }

        // Assign the candidate to the current job
        if ($otysApi->procedures->add($candidateUid, $vacancyUid, $procedureArgs)) {
            // Loop through all postdata to check if there are files which need to be assigned to the current candidate
            foreach ($fieldValues as $fieldName => $postQuestionAnswer) {
                // Check if current answer has documents
                if (isset($postQuestionAnswer['documents'])) {
                    // Loop through all documents and attach them to the current applying candidate
                    foreach ($postQuestionAnswer['documents'] as $file) {
                        $otysApi->documents->attachDocument([
                            "subject" => $file['name'],
                            "typeId" => ($fieldName == 'cvFileName') ? "1" : "201", // type 1 = CV, type 201 = Other
                            "private" => "false",
                            "customerRightsLevel" => 1,
                            "alwaysOnTop" => "false",
                            "applicationUid" => $vacancy['internalId'],
                            "candidateUid" => $candidateUid,
                            "fileUid" => $file['ouid']
                        ]);
                    }
                }
            }

            /**
             * Notify candidate by email that the application was succesfull
             */
            $documentArgsCandidateNotification = [
                'relatedEntities' =>  [
                    [
                        "entityId" => 1, /* vacancy */
                        "recordId" => "{$vacancyUid}"
                    ],
                    [
                        "entityId" => 2, /* candidate */
                        "recordId" => "{$candidateUid}"
                    ],
                ]
            ];

            // If no candidate existed before (equals empty)
            $documentUid = empty($candidate) ? 
            intval(get_option('otys_option_document_template_apply_notify_new_candidate', 0)) :
            intval(get_option('otys_option_document_template_apply_notify_candidate', 0));

            if (empty($candidate)) {
                $webuser = $otysApi->candidates->createWebuser($candidateUid, $fieldValues['Person.emailPrimary']);

                $webuserPassword = isset($webuser['password']) ? $webuser['password'] : '';

                $documentArgsCandidateNotification["answers"] = [
                    "password" => "{$webuserPassword}"
                ];
            }

            // If documentuid equals 0 we won't send the candidate a notification email
            if ($documentUid !== 0) {
                // Get the document used for the candidate notification email
                $document = $otysApi->documents->getDocument($documentUid, $documentArgsCandidateNotification);

                if (!empty($document)) {
                    // Send the notifcation email to the candidate
                    $otysApi->mail->send([
                        "subject" => $document['subject'],
                        "htmlMessage" => $document['htmlBody'],
                        "To" => [
                            $fieldValues['Person.emailPrimary']
                        ]
                    ]);
                }
            }

             /**
             * Notify consultant by email that a candidate applied
             */
            $documentUid = intval(get_option('otys_option_document_template_apply_notify_consultant', 0));
            
            // If documentuid equals 0 we won't send the candidate a notification email
            if ($documentUid !== 0) {
                // Get the document used for the candidate notification email
                $document = $otysApi->documents->getDocument($documentUid, [
                    'relatedEntities' =>  [
                        [
                            "entityId" => 1, /* vacancy */
                            "recordId" => "{$vacancyUid}"
                        ],
                        [
                            "entityId" => 2, /* candidate */
                            "recordId" => "{$candidateUid}"
                        ]
                    ],
                    "answers" => [
                        "password" => "{$webuserPassword}"
                    ]
                ]);

                if (!empty($document)) {
                    // Send the notifcation email to the candidate
                    $otysApi->mail->send([
                        "subject" => $document['subject'],
                        "htmlMessage" => $document['htmlBody'],
                        "To" => [
                            $vacancy['userEmail']
                        ]
                    ]);
                }
            }

            // Change completed status in session to true, to prevent the candidate from applying again
            unset($_SESSION['questionset'][$vacancyUid]);
            $_SESSION['questionset'][$vacancyUid]['completed'] = true;

            return true;
        }
        
        return false;
    }

    /**
     * Succes page for vacancy application
     *
     * @return void
     * @since 1.0.0
     */
    public function success(): void
    {
        $otysApi = new OtysApi();

        $vacancyUid = $otysApi->vacancies->getVacancyUidFromUrl(1);

        $applicationCompleted = $otysApi->candidateQuestionSets->applicationCompleted($vacancyUid);

        if (!$applicationCompleted) {
            wp_redirect(home_url());
            exit();
        }

        $vacancy = $otysApi->vacancies->get($vacancyUid);

        $this->parseArgs('vacancy', $vacancy);
        $this->setTemplate('vacancies/vacancies-success.php');
    }

    /**
     * Confirm email adres
     *
     * @return void
     * @since 1.0.0
     */
    public function confirmEmail(): void
    {
        global $wp;

        $otysApi = new OtysApi();

        $vacancyUid = $otysApi->vacancies->getVacancyUidFromUrl(1);
        $vacancy = $otysApi->vacancies->get($vacancyUid);
        $website = intval(get_option('otys_option_website'), false);
        $questionsetId = $otysApi->candidateQuestionSets->getVacancyQuestionsetId($vacancyUid);
        
        $postedData =  $otysApi->candidateQuestionSets->getFieldValues($vacancyUid, $questionsetId);

        $data = $otysApi->candidateQuestionSets->getData($vacancyUid);


        // Get questionset
        $questionset = $otysApi->candidateQuestionSets->getQuestionSet($questionsetId);

        $errors = [];
        $confirmationcode = $data['code'];

        if (empty($vacancy) || ($website && !array_key_exists($website, $vacancy["publishedWebsites"])) || !$vacancy["published"]) {
            Routes::throwError(404);
        }

        /**
         * Validate the questionset
         */
        foreach ($questionset['pages'] as $page) {
            // Get previous page errors
            $validation = $otysApi->candidateQuestionSets->validate(
                $page['questions'],
                $otysApi->candidateQuestionSets->getPageData($vacancyUid, $page['number'])
            );

            // If the previous page has errors link to that page & display errors
            if (!empty($validation['errors'])) {

                $url = home_url($wp->request);
                $previousPageUrls = add_query_arg(['page-number' => $page['number']], dirname($url) . '/apply/');

                wp_redirect($previousPageUrls . '&displayerrors=1');
                exit;
            }
        }

        /**
         * Handle confirmation
         */
        if (!empty($_POST)) {
            $_POST = filter_input_array(INPUT_POST, FILTER_SANITIZE_FULL_SPECIAL_CHARS);

            /**
             * Verify Recaptcha
             */
            $recaptcha = new Recaptcha();
            $recaptchaResponse = is_string($_POST['g-recaptcha-response']) ? preg_replace("/[^\w-]*$/", "", $_POST['g-recaptcha-response']) : '';

            if (!$recaptcha->verify($recaptchaResponse)) {
                $errors['recaptcha'] = __('reCAPTCHA is incorrect. Please try again.', 'otys-jobs-apply');
            }

            /**
             * Check confirmation code
             */
            if ($_POST['confirmationcode'] != $confirmationcode) {
                $errors['confirmationcode'] = true;
            }

            /**
             * If the confirmation code is correct and there are no errors handle the application
             */
            if (empty($errors)) {
                /**
                 * Handle the application
                 */
                if ($this->doApplication($vacancyUid)) {
                    $thankYouPage = Routes::get('vacancy-apply-success', [
                        'name' => sanitize_title($vacancy['title']) . '-' . $vacancy['uid']
                    ]);

                    wp_redirect(esc_url_raw($thankYouPage));
                    exit;
                } else {
                    echo __('Something went wrong, please contact ths server admin', 'otys-jobs-apply');
                }
            }
        }

        $this->setTemplate('vacancies/vacancies-confirm-email.php');
        $this->parseArgs('data', $postedData);
        $this->parseArgs('vacancy', $vacancy);
        $this->parseArgs('errors', $errors);
    }

    /**
     * Callback [otys-vacancies-filters] shortcode
     *
     * @param array $atts
     * @param array $args
     * @return void
     * @since 1.0.0
     */
    public function search($atts = [], array $args = []): void
    {
        $search = (string) (isset($_GET['search']) && is_string($_GET['search']) ? preg_replace('/[^a-zA-Z0-9 .,&\*\']/', '', $_GET['search']) : '');

        $this->parseArgs('search', $search);
        $this->parseArgs('action', Routes::get('vacancies'));
        $this->parseArgs('matchCriteriaList', $this->model->getMatchCriteriaList());
        
        $this->setTemplate('vacancies/vacancies-search.php');
    }

    /**
     * Ajax entry point for ajax calls
     *
     * @return void
     * @since 1.0.0
     */
    public function ajax(): void
    {
        $otysApi = new OtysApi();

        $_GET = filter_input_array(INPUT_GET, FILTER_SANITIZE_FULL_SPECIAL_CHARS);

        if (!isset($_GET['uid'])) {
            $this->jsonResponse([
                'type' => 'error',
                'code' => 'no_uid',
                'message' => __('No context vacancy UID given', 'otys-jobs-apply')
            ]);
        }

        // Get UID and sanatize
        $vacancyUid = filter_var($_GET['uid'], FILTER_CALLBACK, array('options' => [$this, 'sanitizeOUID']));

        if (!isset($_GET['action'])) {
            $this->jsonResponse([
                'type' => 'failed',
                'code' => 'no_action',
                'message' => __('No action is given.', 'otys-jobs-apply')
            ]);
        }

        /**
         * Delete value
         */
        if ($_GET['action'] == 'delete-data') {
            $type = isset($_GET['type']) ? sanitize_title($_GET['type']) : null;
            $questionId = isset($_GET['question-id']) ? sanitize_title($_GET['question-id']) : null;
            $key = isset($_GET['key']) ? filter_var($_GET['key'], FILTER_CALLBACK, array('options' => [$this, 'sanitizeOUID'])) : null;

            if ($removeResponse = $otysApi->candidateQuestionSets->removeData($vacancyUid, $questionId, $key, $type)) {
                $this->jsonResponse([
                    'type' => 'success',
                    'code' => 'questionset_remove',
                    'message' => __('Data removed succesfully from qusetionset', 'otys-jobs-apply'),
                    'data' => [
                        'id' => esc_attr($key),
                        'questionId' => esc_attr($questionId)
                    ]
                ]);
            } else {
                $this->jsonResponse([
                    'type' => 'error',
                    'code' => 'questionset_remove',
                    'message' => __('Data could not be removed', 'otys-jobs-apply'),
                    'data' => [
                        'id' => esc_attr($key),
                        'questionId' => esc_attr($questionId)
                    ]
                ]);
            }
        }

        $this->jsonResponse([
            'type' => 'error',
            'code' => 'unknown action',
            'message' => __('Unknown action given.', 'otys-jobs-apply')
        ]);
    }
}
