<?php

/**
 * Cache using Wordpress Transients API
 * @package otys-jobs-apply
 */

namespace Otys\OtysPlugin\Includes\Core;

use Otys\OtysPlugin\Includes\OtysApi\OtysApi;

class Cache
{
    public function __construct()
    {
    }

    /**
     * Add cache using set_transient from the Wordpress Transient API
     *
     * @param  mixed $modulename is used for to keep track at which module the cache belongs to
     * @param  mixed $args is used to keep track if a call is the same as the previous appendix can
     * for example be a key generated based on the arguments given
     * @param  mixed $time
     *
     * @return void
     *
     * @since 1.0.0
     */
    public static function add($name, $request, $value, int $time = 600): void
    {
        if ($value === null) {
            return;
        }

        $cacheObject = [
            'request' => $request,
            'value' => $value
        ];

        set_transient(self::getTransientName($name, $request), json_encode($cacheObject), $time);
    }

    /**
     * Generates a appendix based on the value. Expects array or string
     *
     * @param $value
     *
     * @return string
     *
     * @since 1.0.0
     */
    public static function getAppendix($value): string
    {
        if (is_string($value) && !empty($value) && substr($value, 0, 1) == '{') {
            // If the first chracter is a curly bracket we assume the value is json
            $appendix = sha1(sanitize_key($value, true));
        } else if(is_array($value)) {
            $appendix = sha1(sanitize_key(json_encode($value), true));
        } else {
            $appendix = sanitize_key(sha1($value));
        }


        return $appendix;
    }

    /**
     * Generate transient name based on the module name, action and a appendix
     * a module name is atleast required to generate a transient name
     *
     * @param  string $name
     * @param  mixed $value
     *
     * @return string
     *
     * @since 1.0.0
     */
    public static function getTransientName(string $name, $request = null): string
    {
        $transient_prefix = 'otys_' . get_locale();

        $transientName = $transient_prefix;

        if ($name !== '') {
            $transientName .= '_' . $name;
        }

        if ($request !== null) {
            $transientName .= '_' . self::getAppendix($request);
        }

        if ($transient_prefix === $transientName) {
            return false;
        }

        return $transientName;
    }

    /**
     * Get transient (cache) if it exists otherwise return false
     *
     * @param  string $modulename - module name used as indentifier for the transient
     * @param  string $action - action within the module used as indentifier for the transient
     * @param  mixed $appendix - actions for example a list of arguments used as indentifier for the transient
     * may either be a string or an array
     *
     * @return mixed
     * @since 1.0.0
     */
    public static function get(string $name, $request)
    {
        // Check if a transientname can be generated if not return false if true check if the transient exists
        if ($transientName = self::getTransientName($name, $request)) {
            if ($transient = get_transient($transientName)) {
                return json_decode($transient, true);
            }

            return false;
        }

        return false;
    }

    /**
     * Get all plugin cache
     *
     * Returns all cache belonging to the plugin
     *
     * @return array
     * @since 1.0.0
     */
    public static function getList(array $args = [])
    {
        global $wpdb;

        $args = wp_parse_args($args, [
            'keywords' => [],
            'includeExpired' => true
        ]);

        $sql =  "SELECT `option`.`option_id`, `option`.`autoload`, `option`.`option_name` AS `name`, `option`.`option_value` AS `value`, `expire`.`option_value` AS `expire`
        FROM {$wpdb->options} AS `option`
        LEFT JOIN {$wpdb->options} AS `expire` ON `expire`.option_name =
        REPLACE(`option`.option_name, '_transient_otys', '_transient_timeout_otys')
        WHERE `option`.`option_name` LIKE '%transient_otys%' ";
        
        
        // Check if we should include expired transients
        if ($args['includeExpired'] === false) {
            $sql .= " AND `expire`.`option_value` > UNIX_TIMESTAMP() ";
        }
        
        // Keyword search
        if (!empty($args['keywords']) && is_array($args['keywords'])) {
            $sql .= "AND (";

            foreach ($args['keywords'] as $key => $keyword) {
                if ($key === 0) {
                    $sql .= "`option`.`option_value` LIKE '%" . $keyword . "%' ";
                    continue;
                }
                $sql .= " OR `option`.`option_value` LIKE '%" . $keyword . "%' ";
            }
            
            $sql .= ")";
        }
        
        $sql .= " ORDER BY `expire` DESC";

        return $wpdb->get_results($wpdb->prepare($sql), ARRAY_A);
    }

    /**
     * Deletes all cache belong to the plugin
     * @return void
     * @since 1.0.0
     */
    public static function deleteAll(): void
    {
        $cache = static::getList();

        foreach ($cache as $transient) {
            $transient_name = str_replace('_transient_otys', 'otys', $transient['name']);
            delete_transient($transient_name);
        }
    }

    /**
     * Refresh cache based on transients
     * @since 1.0.0
     */
    public static function refresh(array $transients = [])
    {
        $otysApi = new OtysApi;
        $responses = [];

        // First remove all cache
        foreach ($transients as $transient) {
            $transientName = str_replace('_transient_otys', 'otys', $transient['name']);

            $responses[$transientName] = delete_transient($transientName);
        }

        // Refresh all cache
        foreach ($transients as $transientKey => $transient) {
            $transientValue = json_decode($transient['value'], true);
            $transientRequest = $otysApi->createRequest($transientValue['request']);

            $otysApi->post($transientRequest, true, true);
        }

        return $responses;
    }

    /**
     * Search for transient
     *
     * Uses LIKE to find transient and matches the part of the search string
     *
     * Note that this only applies to transients which contain _transient_otys to avoid affecting
     * options and only effect transients of this plugin
     *
     * @param string $search
     * @return array
     * @since 1.0.0
     */
    public static function search(string $transient, $exact = false)
    {
        global $wpdb;

        $search = esc_sql($transient);

        $where = $exact ?
        " WHERE `option`.`option_name` LIKE ('%_transient_otys%') AND `option`.`option_name` = '{$search}'"
        : "WHERE `option`.`option_name` LIKE ('%_transient_otys%') AND `option`.`option_name` LIKE '%{$search}%' ";

        $sql  ="
        SELECT `option`.`autoload`, `option`.`option_name` AS `name`, `expire`.`option_value` AS `expire`, `option`.`option_value` AS `value`
        FROM {$wpdb->options} AS `option`
        LEFT JOIN {$wpdb->options} AS `expire` ON `expire`.option_name =
        REPLACE(`option`.`option_name`, '_transient_otys', '_transient_timeout_otys')
        {$where}
        ORDER BY `expire` DESC";

        return $wpdb->get_results($wpdb->prepare($sql), ARRAY_A);
    }

    /**
     * Deletes all transients that match the transient
     *
     * Note that this only applies to transients which contain _transient_otys to avoid affecting
     * options and only effect transients of this plugin
     *
     * @param string $transient
     * @return mixed
     * @since 1.0.0
     */
    public static function delete(string $transientName, $exact = false)
    {
        global $wpdb;

        $transient = esc_sql($transientName);

        $where = $exact
        ? "WHERE `option_name` LIKE ('%_transient_otys%') AND `option_name` = '{$transientName}'"
        : "WHERE `option_name` LIKE ('%_transient_otys%') AND `option_name` LIKE ('%{$transientName}%')";

        $sql = "SELECT * FROM `{$wpdb->options}` {$where}";

        $transients = $wpdb->get_results($wpdb->prepare($sql), ARRAY_A);

        $deleted = [];

        foreach ($transients as $transient) {
            $name = str_replace('_transient_', '', $transient['option_name']);
  
            $deleted[$name] = delete_transient($name);
        }

        return $deleted;
    }
}
