<?php

/**
 * The core Plugin class
 * @since 1.0.0
 * @package otys-jobs-apply
 */

namespace Otys\OtysPlugin\Includes\Core;

use Otys\OtysPlugin\Includes\OtysApi\OtysApi;

class OtysPlugin extends Base
{
    /**
     * Instances
     */
    public $hooks;
    public $shortcodes;
    public $webhooks;
    public $language;
    public $routes;
    public $adminPages;
    public $enqueue;
    public $hasValidSession;
    public $logs;

    /**
     * Load the core functionality of the plugin
     *
     * @since 1.0.0
     */
    public function __construct()
    {
        $this->registerBaseHooks();

        $otysApi = new OtysApi;

        $this->hasValidSession = $otysApi->check();

        /**
         * Init plugin modules
         */

        /**
         * @var \Otys\OtysPlugin\Includes\Core\Hooks
         */
        $this->hooks = Hooks::getInstance();

        /**
         * @var \Otys\OtysPlugin\Includes\Core\Logs
         */
        $this->logs = Logs::getInstance();

        /**
         * @var \Otys\OtysPlugin\Includes\Core\Shortcodes
         */
        $this->shortcodes = Shortcodes::getInstance();

        /**
         * @var \Otys\OtysPlugin\Includes\Core\Webhooks
         */
        $this->webhooks = Webhooks::getInstance();

        /**
         * @var \Otys\OtysPlugin\Includes\Core\Language
         */
        $this->language = Language::getInstance();

        /**
         * @var \Otys\OtysPlugin\Includes\Core\Routes
         */
        $this->routes = Routes::getInstance();

        /**
         * @var \Otys\OtysPlugin\Includes\Core\AdminPAges
         */
        $this->adminPages = AdminPages::getInstance();

        /**
         * @var \Otys\OtysPlugin\Includes\Core\Enqueue
         */
        $this->enqueue = Enqueue::getInstance();

        // Start session
        Hooks::addAction('init', $this, 'initSession', 1, 0);

        // Register sidebars
        Hooks::addAction('init', $this, 'registerSidebars', 10, 0);

        // Register widgets
        Hooks::addAction('widgets_init', $this, 'registerWidgets', 10, 0);

        // Add admin enqueue
        Hooks::addAction('admin_enqueue_scripts', $this, 'enqueueAdmin', 10, 0);

        if ($this->hasValidSession) {
            // Display what pages are used for in the pages overview
            Hooks::addFilter('display_post_states', $this, 'filterPostStates', 10, 2);

            /**
             * Init shortcode classes
             */
            new \Otys\OtysPlugin\Shortcodes\VacanciesShortcodes;
            new \Otys\OtysPlugin\Shortcodes\RoutesShortcodes;
            new \Otys\OtysPlugin\Shortcodes\ClientShortcodes;
        }

        // Add shortcodes
        $this->shortcodes->init();

        // Run the before init function
        $this->routes->runInitCallback();

        // Hooks & Shortcodes must be initialized last
        $this->hooks->init();
    }

    public function filterPostStates($states, $post)
    {
        // TODO make it so this function is not needed and save post state within the post itself
        if (is_admin()) {
            $moduleList = [
                'vacancies' => __('Vacancies overview', 'otys_ows'),
                'vacancy' => __('Vacancy detail', 'otys_ows')
            ];

            /**
             * For otys_option_language_routes save the slug of the page aswell
             */
            $routes = get_option('otys_option_language_routes', "");
            if ($routes !== "") {
                foreach ($routes as $moduleName => $moduleValues) {
                    foreach ($moduleValues as $key => $moduleValue) {
                        if (isset($moduleValue['slug']) && $moduleValue['slug'] == $post->post_name) {
                            $states['otys_route_' . $moduleName] = $moduleList[$moduleName];
                        }
                    }
                }
            }
        }

        return $states;
    }

    /**
     * enqueue Admin scripts
     *
     * @return void
     */
    public function enqueueAdmin()
    {
        wp_enqueue_editor();
        wp_enqueue_script('otys-widget', OTYS_PLUGIN_ASSETS_URL . '/javascript/admin/widgets.js', array('wp-tinymce', 'jquery'), false, true);
        wp_enqueue_script('otys-admin', OTYS_PLUGIN_ASSETS_URL . '/javascript/admin/admin.js', array('jquery'), false, true);
        wp_enqueue_style('otys-admin', OTYS_PLUGIN_ASSETS_URL . '/css/admin/admin.css');
    }

    /**
     * Register base hooks needed for the plugin to work
     *
     * @return void
     *
     * @since 1.0.0
     */
    public function registerBaseHooks(): void
    {
        register_activation_hook(OTYS_PLUGIN_FILE_URL, [$this, 'activate']);
        register_deactivation_hook(OTYS_PLUGIN_FILE_URL, [$this, 'deactivate']);
        register_uninstall_hook(OTYS_PLUGIN_FILE_URL, [__CLASS__, 'uninstall']);
    }

    /**
     * Plugin activation
     *
     * @return void
     * since 1.0.0
     */
    public function activate(): void
    {
        Logs::createTable();

        // Set default value of settings
        if (get_option('otys_option_recaptcha_threshold') === false) {
            update_option('otys_option_recaptcha_threshold', 0.5);
        }

        // Set default vacancies per page number
        if (get_option('otys_option_vacancies_per_page') === false) {
            update_option('otys_option_vacancies_per_page', 10);
        }

        $otysApi = new OtysApi();
        if ($otysApi->check()) {
            Webhooks::registerWebhooks();
        }
    }

    /**
     * Plugin deactivation
     *
     * @return void
     *
     * @since 1.0.0
     */
    public function deactivate(): void
    {
        Cache::deleteAll();

        Webhooks::deleteWebhooks();
    }

    /**
     * Plugin uninstall
     *
     * @return void
     * @since 1.0.0
     */
    public static function uninstall(): void
    {
        global $wpdb;

        Logs::deleteTable();

        Webhooks::deleteWebhooks();

        // Delete all OTYS Options
        $sql = "DELETE FROM $wpdb->options WHERE option_name LIKE '%otys_option%'";
        $wpdb->get_results($wpdb->prepare($sql));

        Cache::deleteAll();
    }

    /**
     * Plugin initialisation
     *
     * @return void
     * @since 1.0.0
     */
    public function initSession(): void
    {
        /**
         * Start session if there's no session yet
         */
        if (session_id() == '' || !isset($_SESSION) || session_status() === PHP_SESSION_NONE) {
            session_start([
                'cookie_lifetime' => 86400,
            ]);
        }

        /**
         * Add UTM tags to session
         */
        $_SESSION['utm_tags'] = isset($_SESSION['utm_tags']) ? filter_var($_SESSION['utm_tags'], FILTER_CALLBACK, array('options' => [$this, 'sanitizeUTM'])) : [];

        /**
         * Search get parameters for utm information and add to the session and cookies
         */
        if (isset($_GET['utm_source'])) {
            $utmSource = filter_var($_GET['utm_source'], FILTER_CALLBACK, array('options' => [$this, 'sanitizeUTM']));

            if ($utmSource !== '') {
                $_SESSION['utm_tags']['source'] = $utmSource;
                setcookie("utm_source", $utmSource, strtotime('+30 days'), '/', esc_url_raw($_SERVER['HTTP_HOST']));
            }
        }

        if (isset($_GET['utm_medium'])) {
            $utmMedium = filter_var($_GET['utm_medium'], FILTER_CALLBACK, array('options' => [$this, 'sanitizeUTM']));

            if ($utmMedium !== '') {
                $_SESSION['utm_tags']['medium'] = $utmMedium;
                setcookie("utm_medium", $utmMedium, strtotime('+30 days'), '/', esc_url_raw($_SERVER['HTTP_HOST']));    
            }
        }

        if (isset($_GET['utm_campaign'])) {
            $utmCampaign = filter_var($_GET['utm_campaign'], FILTER_CALLBACK, array('options' => [$this, 'sanitizeUTM']));

            if ($utmCampaign !== '') {
                $_SESSION['utm_tags']['campaign'] = $utmCampaign;
                setcookie("utm_campaign", $utmCampaign, strtotime('+30 days'), '/', esc_url_raw($_SERVER['HTTP_HOST']));
            }
        }

        if (isset($_GET['utm_term'])) {
            $utmTerm =  filter_var($_GET['utm_term'], FILTER_CALLBACK, array('options' => [$this, 'sanitizeUTM']));

            if ($utmTerm !== '') {
                $_SESSION['utm_tags']['term'] = $utmTerm;
                setcookie("utm_term", $utmTerm, strtotime('+30 days'), '/', esc_url_raw($_SERVER['HTTP_HOST']));
            }
        }

        if (isset($_GET['utm_content'])) {
            $utmContent = filter_var($_GET['utm_content'], FILTER_CALLBACK, array('options' => [$this, 'sanitizeUTM']));

            if ($utmContent !== '') {
                $_SESSION['utm_tags']['content'] = $utmContent;
                setcookie("utm_content", $utmContent, strtotime('+30 days'), '/', esc_url_raw($_SERVER['HTTP_HOST']));
            }
        }
    }

    /**
     * Register plugin specific sidebars
     *
     * @return void
     *
     * @since 1.0.0
     */
    public function registerSidebars(): void
    {
        register_sidebar(
            array(
                'id' => 'vacancy_detail_footer',
                'name' => __('Vacancy detail footer'),
                'description' => __('The vacancy detail footer is content displayed under the vacancy detail text'),
                'before_widget' => '<div id="%1$s" class="widget %2$s">',
                'after_widget' => '</div>',
                'before_title' => '<h3 class="widget-title">',
                'after_title' => '</h3>',
            )
        );
    }

    public function registerWidgets(): void
    {
        // /**
        //  * Register Consultancy Widget
        //  */
        // register_widget('Otys\OtysPlugin\Widgets\ConsultantWidget\ConsultantWidget');

        // /**
        //  * Register Vacancy Shortlist Widget
        //  */
        // register_widget('Otys\OtysPlugin\Widgets\VacancyShortlistWidget\VacancyShortlistWidget');
    }
}
