<?php

namespace Otys\OtysPlugin\Includes\Core;

class Routes extends Base
{
    private static $routes = array();

    public function __construct()
    {
        Hooks::addAction('init', $this, 'initRewriteRules', 1, 1);
        Hooks::addFilter('query_vars', $this, 'addQueryVars', 2, 1);
        Hooks::addFilter('template_include', $this, 'checkRewrites', 3, 1);
        Hooks::addFilter('get_canonical_url', $this, 'checkCanonical', 3, 1);
        Hooks::addAction('template_redirect', $this, 'checkPage', 1);
    }

    /**
     * Check chanonical changes the canonical for page pages.
     * when creating a non existing page wordpress sets the canonical to
     * be the pageid=-99, which we ofcouse do not want. Therefore we will
     * be changing the canonical for fake pages.
     *
     * @param [type] $canonical
     * @return string
     * @since 1.0.0
     */
    public function checkCanonical($canonical): string {
        global $post;
        
        /**
         * If the current page is a fake post prevent Wordpress from
         * creating a canonical
         */
        if ($post->ID == -99) {
            return false;
        }
        
        return $canonical;
    }

    /**
     * Check rewrite and generate fake page if page does not actually exists.
     *
     * @return void
     *
     * @version 1.0.0
     */
    public function checkPage(): void
    {
        if ($route = self::getCurrentRoute()) {
            if (!is_page()) {
                $this->createFakePage($route);
            }
        }
    }
    
    /**
     * Get rewrite by slug
     *
     * @param string $slug
     * @return string
     */
    public static function get(string $slug, $replacements = []): string
    {
        $locale = get_locale();

        /**
         * Try getting the route from the routes file
         */
        foreach (static::$routes as $route) {
            if ($route['slug'] === $slug && ($route['locale'] === $locale || $route['locale'] === "")) {
                $value = $route['route'];

                foreach ($replacements as $search => $replacement) {
                    $value = str_replace(':' . $search, $replacement, $value);
                }

                return $value;
            }
        }

        /**
         * Try getting the route from the database
         */
        $routes = get_option('otys_option_language_routes');

        if ($slug && $routes) {
            if (!empty($routes) && !empty($routes[$slug])) {
                foreach ($routes[$slug] as $route) {
                    if ($locale == $route['locale']) {
                        return home_url($route['slug']);
                    }
                }
            }
        }

        return '';
    }

    /**
     * Undocumented function
     *
     * @return void
     * @since 1.0.0
     */
    public function createFakePage($route = [])
    {
        global $wp, $wp_query;
        
        // Init vars
        $pageType = (isset($route['type']) ? $route['type'] : 'page');
        // Set status header
        status_header(200);
        
        // Create our fake post
        $post_id = -99;

        $post = new \stdClass();
        $post->ID = $post_id;
        $post->post_author = 1;
        $post->post_date = current_time('mysql');
        $post->post_date_gmt = current_time('mysql', 1);
        $post->post_title = 'OTYS Page';
        $post->post_content = 'This is a OTYS page';
        $post->post_status = 'publish';
        $post->comment_status = 'closed';
        $post->ping_status = 'closed';
        //$post->post_name = 'fake-page-' . rand(1, 99999); // append random number to avoid clash
        $post->post_type = $pageType;
        $post->filter = 'raw'; // important!

        $wp_post = new \WP_Post($post);

        // Update the main query
        $wp_query->post = $wp_post;
        $wp_query->posts = array($wp_post);
        $wp_query->queried_object = $wp_post;
        $wp_query->queried_object_id = $post_id;
        $wp_query->found_posts = 1;
        $wp_query->post_count = 1;
        $wp_query->max_num_pages = 1;
        $wp_query->is_page = true;
        $wp_query->is_singular = true;
        $wp_query->is_single = false;
        $wp_query->is_attachment = false;
        $wp_query->is_archive = false;
        $wp_query->is_category = false;
        $wp_query->is_tag = false;
        $wp_query->is_tax = false;
        $wp_query->is_author = false;
        $wp_query->is_date = false;
        $wp_query->is_year = false;
        $wp_query->is_month = false;
        $wp_query->is_day = false;
        $wp_query->is_time = false;
        $wp_query->is_search = false;
        $wp_query->is_feed = false;
        $wp_query->is_comment_feed = false;
        $wp_query->is_trackback = false;
        $wp_query->is_home = false;
        $wp_query->is_embed = false;
        $wp_query->is_404 = false;
        $wp_query->is_paged = false;
        $wp_query->is_admin = false;
        $wp_query->is_preview = false;
        $wp_query->is_robots = false;
        $wp_query->is_posts_page = false;
        $wp_query->is_post_type_archive = false;

        wp_cache_add($post_id, $wp_post, 'posts');

        $GLOBALS['wp_query'] = $wp_query;
        $wp->register_globals();
    }

    /**
     * ADd a route
     * @param  string $slugUID Unique slug as identifier
     * @param  string $route
     * @param  mixed $class
     * @param  mixed $function
     * @param  callable 
     * @return void
     * @since 1.0.0
     */
    public static function add(string $slug = '', array $urls, $function, $methods, string $locale = '', string $type = 'page', callable $initCallback = NULL)
    {
        foreach ($urls as $urlLanguage => $url) {
            $routeLanguage = static::getLanguage($locale);

            $url = wp_make_link_relative(get_site_url(null)) . $url;

            if (array_key_exists($routeLanguage, $urls) && $routeLanguage != $urlLanguage) {
                continue;
            }

            $regexPattern = "^" . preg_replace('/\\\:[a-zA-Z0-9\_\-]+/', '([a-zA-Z0-9\-\_]+)', preg_quote($url)) . "$";

            static::$routes[] = [
                'slug' => $slug,
                'route' => $url,
                'rewrite_rule' => $regexPattern,
                'function' => $function,
                'method' => $methods,
                'locale' => $locale,
                'type' => $type,
                'initCallback' => $initCallback
            ];
        }
    }

    /**
     * addQueryVars adds otys-page to the allowed query vars list
     *
     * @param  array $vars
     * @return array
     * @since 1.0.0
     */
    public static function addQueryVars($vars)
    {
        // Allow the query var ?otys-page
        $vars[] = 'otys';
        $vars[] = 'page-number';

        return $vars;
    }

    /**
     * Add rewrite rules to wordpress
     *
     * @return void
     * @since 1.0.0
     */
    public static function initRewriteRules()
    {
        foreach (static::$routes as $route) {
            add_rewrite_rule($route['rewrite_rule'], 'index.php?otys=1', 'top');
        }
    }

    /**
     * Run init callback for the current route
     *
     * @return void
     * @since 1.0.0
     */
    public function runInitCallback(): void
    {
        foreach (static::$routes as $route) {
            // Check if the current url matches the route
            if (static::checkRoute($route)) {
                // If the url matches the current route then we run the before init function
                if (is_callable($route['function'][0]::initCallback($route))) {
                    $route['function'][0]::initCallback();
                }
            }
        }
    }

    /**
     * Check if current URL equals a custom route
     *
     * @param  mixed $path
     * @return void
     * @since 1.0.0
     */
    public static function checkRewrites($template)
    {
        if ($route = self::getCurrentRoute()) {
            /**
             * Check if the current route has the same locale as the current locale
             * if not then we will change the locale
             */
            if (get_locale() != $route['locale']) {
                switch_to_locale($route['locale']);
            }

            if (class_exists($route['function'][0]) && is_callable([$route['function'][0], 'getInstance'])) {
                $instance = call_user_func([$route['function'][0], 'getInstance']);

                // Check if the function is defined && is a callable and cann it if possible;
                $function = (!array_key_exists("1", $route['function'])) ? false : $route['function'][1];

                if ($function && is_callable($instance->$function())) {
                    call_user_func($instance->$function());
                }

                // Gets the template set in the route function, get template is defined in the BaseModel
                if ($instance::loadTemplate($instance::getTemplate(), $instance::getArgs())) {
                    return false;
                } else {
                    self::throwError();
                }
            }
        }

        return $template;
    }

    /**
     * Get if current url exists in routes
     *
     * @return mixed : Returns false if no route is found or returns route if route is found
     */
    public static function getCurrentRoute()
    {
        $currentMethod = is_string($_SERVER['REQUEST_METHOD']) ? preg_replace("/[^a-zA-Z]/", "", $_SERVER['REQUEST_METHOD']) : '';

        $currentUrl = parse_url(filter_var($_SERVER['REQUEST_URI'], FILTER_SANITIZE_URL), PHP_URL_PATH);
        $currentUrl = (substr($currentUrl, -1) === '/') ? $currentUrl : $currentUrl . '/';

        foreach (self::$routes as $route) {
            if (in_array($currentMethod, $route['method']) && preg_match('#' . $route['rewrite_rule'] . '#', $currentUrl)) {
                return $route;
            }
        }

        return false;
    }

    /**
     * Check route against current REQUEST
     * If route matches the current request return true if
     * route does not match the curren request return false.
     *
     * @param array $route
     * @return boolean
     * @since 1.0.0
     */
    public static function checkRoute(array $route): bool
    {
        $currentMethod = is_string($_SERVER['REQUEST_METHOD']) ? preg_replace("/[^a-zA-Z]/", "", $_SERVER['REQUEST_METHOD']) : '';

        $currentUrl = parse_url(filter_var($_SERVER['REQUEST_URI'], FILTER_SANITIZE_URL), PHP_URL_PATH);
        $currentUrl = (substr($currentUrl, -1) === '/') ? $currentUrl : $currentUrl . '/';

        if (in_array($currentMethod, $route['method']) && preg_match('#' . $route['rewrite_rule'] . '#', $currentUrl)) {
            return true;
        }

        return false;
    }

    /**
     * Throw error page. Loads in 404 page
     *
     * @param string $errorCode
     * @param string $template
     * @param array $args
     * @param boolean $die
     * @return void
     * @since 1.0.0
     */
    public static function throwError($errorCode = '404', $template = '404', $args = [], bool $die = true): void
    {
        // Set status header with set status code
        status_header($errorCode);
        nocache_headers();

        $errors = Errors::get();

        $templatePath = false;
        
        if (($path = self::locateTemplate($template, true)) !== false) {
            $templatePath = $path;
        } else {
            $templatePath = get_query_template('404');
        }

        if ($templatePath !== '') {
            load_template($templatePath, true, [
                'errors' => $errors
            ], $args);
        }

        // Now stop the rest of the script execution
        if ($die != false) {
            die();
        }
    }

    /**
     * locateTemplate
     * Locate template file in theme directories
     *
     * @param string $template
     * @return string
     * @since 1.0.0
     */
    public static function locateTemplate(string $template = '', $checkRoot = false): string
    {
        if ($template === '') {
            return '';
        }

        $extension = pathinfo($template, PATHINFO_EXTENSION);

        // Check if the template has .php at the end if not add .php
        $template = ($extension === 'php') ? $template : $template . '.php';

        if ($file_path = locate_template('otys-jobs-apply/' . $template)) {
            return $file_path;
        } elseif ($checkRoot && $file_path = locate_template($template)) { 
            return $file_path;
        } elseif (is_file(OTYS_PLUGIN_TEMPLATE_URL . '/' . $template)) {
            return OTYS_PLUGIN_TEMPLATE_URL . '/' . $template;
        }
        
        return '';
    }
}
