<?php

namespace Otys\OtysPlugin\Includes\OtysApi;

use WP_Error;

class Documents extends OtysApiBase
{
    /**
     * Add document to OTYS
     *
     * @param array $file
     * @param boolean $required: wheter the file is required
     * @return mixed
     * @since 1.0.0
     */
    public function addDocument(array $file = [])
    {
        $response = $file;
        $response['error'] = false;

        // Check if the request is invalid
        if (!isset($file['error']) || is_array($file['error'])) {
            return new WP_Error('otys_document', sprintf(__('%s is an invalid file', 'otys-jobs-apply'), $file['name']));
        }

        // Check errors
        switch ($file['error']) {
            case UPLOAD_ERR_OK:
                break;
            case UPLOAD_ERR_NO_FILE:
                return new WP_Error('otys_document', __('No file uploaded', 'otys-jobs-apply'));
            case UPLOAD_ERR_INI_SIZE:
                return new WP_Error('otys_document', sprintf(__('File %s exceeds filesize limit', 'otys-jobs-apply'), $file['name']));
            case UPLOAD_ERR_FORM_SIZE:
                return new WP_Error('otys_document', sprintf(__('File %s exceeds filesize limit', 'otys-jobs-apply'), $file['name']));
            default:
                return new WP_Error('otys_document', __('Unknown error occured', 'otys-jobs-apply'));
        }

        // TODO: Max filesize should be equal to the max file size of Go
        $maxFilesize = 10000000;
        $formattedFileSize = $this->formatBytes($maxFilesize);

        if ($file['size'] > $maxFilesize) {
            return new WP_Error(
                'otys_document',
                sprintf(
                    __('File %1$s exceeds filesize limit. Max file size is %2$d', 'otys-jobs-apply'),
                    $file['name'],
                    $formattedFileSize
                )
            );
        }

        // Allowed extensions
        $allowedExtensions = [
            'image/jpeg',
            'image/pjpeg',
            'image/png',
            'image/x-png',
            'image/gif',
            'application/vnd.openxmlformats-officedocument.wordprocessingml.document',
            'application/msword',
            'text/plain',
            'text/rtf',
            'application/pdf',
            'application/octet-stream'
        ];

        // Check upload file type
        $finfo = new \finfo(FILEINFO_MIME_TYPE);

        if (
            ($extension = array_search(
                $finfo->file($file['tmp_name']),
                $allowedExtensions,
                true
            )) === false
        ) {
            return new WP_Error('otys_document', sprintf(__('File extension of %s is not allowed', 'otys-jobs-apply'), $file['name']));
        }

        $boundary = wp_generate_password(24);
        $body = '';
        $body .= '--' . $boundary . "\r\n";
        $body .= 'Content-Disposition: form-data; name="file"; filename="' . basename($file['name']) . "\"\r\n";
        $body .= 'Content-Type: ' . $file['type'] . "\r\n\r\n";
        $body .= file_get_contents($file['tmp_name']) . "\r\n";
        $body .= '--' . $boundary . "\r\n";
        $body .= 'Content-Disposition: form-data; name="sessionId"' . "\r\n";
        $body .= 'Content-Type: application/json' . "\r\n\r\n";
        $body .= $this->otysApi->session . "\r\n";
        $body .= '--' . $boundary . '--' . "\r\n";

        $args = array(
            'body' => $body,
            'timeout'     => '5',
            'redirection' => '5',
            'httpversion' => '1.0',
            'blocking'    => true,
            'cookies'     => array(),
            'headers' => array(
                'accept'       => 'application/json', // The API returns JSON
                'content-type' => 'multipart/form-data;boundary=' . $boundary, // Set content type to multipart/form-data
            )
        );

        $response = wp_remote_post('https://ows.otys.nl/fileService.php', $args);

        if (is_wp_error($response) || !isset($response['body'])) {
            return [];
        }

        $responseBody = json_decode($response['body'], true);

        return reset($responseBody);
    }

    public function textKernalCV($fileUid, $candidateUid)
    {
        $request = [
            "jsonrpc" => "2.0",
            "method" => "Otys.Services.CandidateService.textkernelFromFile",
            "params" => [
                $this->otysApi->session,
                [
                    "fileId" => "YrxpQ2SGI0RxTgYF",
                    "candUid" => "YrxpQ9bVe3p5BZZy"
                ]
            ],
            "id" => 1
        ];

        return $this->post($request);
    }

    /**
     * Add multiple documents
     *
     * @param array $upload
     * @return array
     */
    public function addDocuments(array $upload): array
    {
        $response = [];

        if (is_array($upload['name'])) {
            // Handle multifile upload
            foreach ($upload['name'] as $fileKey => $fileName) {
                $singleFile = [
                    'name'  => $fileName,
                    'type' => $upload['type'][$fileKey],
                    'tmp_name' => $upload['tmp_name'][$fileKey],
                    'error' => $upload['error'][$fileKey],
                    'size' => $upload['size'][$fileKey]
                ];

                $response[] = [
                    'response' => $this->addDocument($singleFile),
                    'file' => $singleFile
                ];
            }
        } else {
            // Handle single file upload
            $response[] = [
                'response' => $this->addDocument($upload),
                'file' => $upload
            ];
        }

        return $response;
    }

    /**
     * Attach document to dossier
     *
     * @param array $options
     * @return mixed
     * @since 1.0.0
     */
    public function attachDocument(array $options = [])
    {
        $options = array_merge([
            'subject' => __('no subject', 'otys-jobs-apply'),
            'typeId' => false,
            'private' => false,
            'customerRightsLevel' => 1,
            'alwaysOnTop' => false,
            'description' => '',
            'fileUid' => false
        ], $options);

        $request = [
            "jsonrpc" => "2.0",
            "method" => "Otys.Services.AttachedDocumentsService.add",
            "params" => [
                $this->otysApi->session,
                $options
            ],
            "id" => 1
        ];

        return $this->post($request, false, false);
    }

    /**
     * Format number to bytes
     *
     * @param mixed $bytes
     * @param integer $precision
     * @return int
     */
    public function formatBytes($bytes, $precision = 2)
    {
        $units = array('B', 'KB', 'MB', 'GB', 'TB');

        $bytes = max($bytes, 0);
        $pow = floor(($bytes ? log($bytes) : 0) / log(1024));
        $pow = min($pow, count($units) - 1);

        return round($bytes, $precision) . ' ' . $units[$pow];
    }

    /**
     * Get document
     *
     * @param integer $id Document ID
     * @param array $args Arguments used to fill in merge fields
     * @return array
     * @since 1.0.0
     */
    public function getDocument(int $uid, array $args = [], $relativeToCurrentLocale = true): array
    {
        $args = wp_parse_args($args, [
            "relatedEntities" => [],
            "answers" => []
        ]);

        if ($relativeToCurrentLocale) {
            if (($translatedUid = $this->getDocumentTranslationUid($uid, $this->getLanguage())) !== 0) {
                $uid = $translatedUid;
            }
        }

        $request = [
            "jsonrpc" => "2.0",
            "method" => "Otys.Services.Sdv2StandardDocumentService.mergeDocument",
            "params" => [
                $this->otysApi->session,
                $uid,
                $args
            ],
            "id" => 1
        ];

        $response = $this->post($request, false, false);

        return $response;
    }

    /**
     * Check if a document exists
     *
     * @param string $name
     * @return void
     */
    public function documentExists(string $name, string $language = ''): int
    {
        $existingRequest = [
            "jsonrpc" => "2.0",
            "method" => "Otys.Services.Sdv2StandardDocumentService.getListEx",
            "params" => [
                $this->otysApi->session,
                [
                    "what" => [
                        "uid" => 1
                    ],
                    "limit" => 1,
                    "offset" => 0,
                    "getTotalCount" => true,
                    "sort" => [
                        "uid" => "DESC"
                    ],
                    "condition" => [
                        "type" => "OR",
                        "items" => [
                            [
                                "type" => "OR",
                                "items" => [
                                    [
                                        "type" => "COND",
                                        "field" => "name",
                                        "op" => "LIKE",
                                        "param" => "%{$name}%"
                                    ]
                                ]
                            ]
                        ]
                    ]
                ]
            ],
            "id" => 1
        ];

        $existResponse = $this->post($existingRequest, false, $language);

        // Return uid of existing snippet if snippet with this name already exists
        if (!empty($existResponse) && is_array($existResponse) && isset($existResponse['listOutput']) && !empty($existResponse['listOutput'])) {
            return $existResponse['listOutput'][0]['uid'];
        }

        return 0;
    }

    /**
     * Create a new document
     *
     * @param string $name
     * @param bool $unique
     * @return mixed
     * @since 1.0.0
     */
    public function createDocument(string $name = '', array $args = [], string $language = '')
    {
        $args = wp_parse_args($args, [
            "name" => "{$name}"
        ]);

        $request = [
            "jsonrpc" => "2.0",
            "method" => "Otys.Services.Sdv2StandardDocumentService.add",
            "params" => [
                $this->otysApi->session,
                $args
            ],
            "id" => 1
        ];

        $response  = $this->post($request, false, $language);

        if (!empty($response) && (is_string($response) || is_int($response))) {
            // TODO remove extra update request from code by making defaultOutput insertable at service
            if (isset($args['defaultOutput']) && is_string($args['defaultOutput'])) {
                $updateRequest = [
                    "jsonrpc" => "2.0",
                    "method" => "Otys.Services.Sdv2StandardDocumentService.update",
                    "params" => [
                        $this->otysApi->session,
                        $response,
                        [
                            "defaultOutput" => "{$args['defaultOutput']}"
                        ]
                    ],
                    "id" => 1
                ];

                $this->post($updateRequest, false, $language);
            }

            return intval($response);
        }

        return 0;
    }

    /**
     * Create a new snippet or returns id of existing snippet
     *
     * @param string    $name       Name of snippet
     * @param bool      $unique     Make sure that name is unique
     * @return mixed
     * @since 1.0.0
     */
    public function createSnippet(string $name = '', string $content = '', $unique = true, string $language = ''): int
    {
        // If snippets must be unique then check if snippet already exists
        if ($unique === true) {
            $existRequest = [
                "jsonrpc" => "2.0",
                "method" => "Otys.Services.Sdv2SnippetService.getListWithParentsEx",
                "params" => [
                    $this->otysApi->session,
                    [
                        "what" => [
                            "uid" => 1,
                            "parentUid" => true,
                            "childrenCount" => 1,
                            "title" => 1,
                            "category" => 1,
                            "categoryId" => 1
                        ],
                        "limit" => 1,
                        "offset" => 0,
                        "getTotalCount" => 1,
                        "condition" => [
                            "type" => "OR",
                            "items" => [
                                [
                                    "type" => "OR",
                                    "items" => [
                                        [
                                            "type" => "COND",
                                            "field" => "title",
                                            "op" => "LIKE",
                                            "param" => "%{$name}%"
                                        ]
                                    ]
                                ]
                            ]
                        ],
                        "getListTotals" => false,
                        "excludeLimitCheck" => true,
                        "sort" => [
                            "uid" => "ASC"
                        ]
                    ],
                    0
                ],
                "id" => 1
            ];

            $existResponse = $this->post($existRequest, false, $language);

            // Return uid of existing snippet if snippet with this name already exists
            if (!empty($existResponse) && is_array($existResponse) && isset($existResponse['listOutput']) && !empty($existResponse['listOutput'])) {
                return $existResponse['listOutput'][0]['uid'];
            }
        }

        // If snippet does not exist, then we create a new snippet and return the uid
        $request = [
            "jsonrpc" => "2.0",
            "method" => "Otys.Services.Sdv2SnippetService.add",
            "params" => [
                $this->otysApi->session,
                [
                    "title" => "{$name}",
                    "categoryId" => 0,
                    "content" => "{$content}",
                    "languageCode" => "nl"
                ]
            ],
            "id" => 1
        ];

        $response =  $this->post($request, false, $language);

        if (is_string($response) || is_int($response)) {

            return intval($response);
        }

        return 0;
    }

    /**
     * Get document Multilangual OUID
     *
     * @param int $documentId
     * @return string
     */
    public function getDocumentMultiLangOuid(int $documentId, string $language = ''): string
    {
        $request = [
            "jsonrpc" => "2.0",
            "method" => "Otys.Services.Sdv2StandardDocumentService.fetchMultiLangOuid",
            "params" => [
                $this->otysApi->session,
                $documentId
            ],
            "id" => 1
        ];

        $response = $this->post($request, false, $language);

        if (is_string($response)) {
            return $response;
        }

        return '';
    }

    /**
     * Add snippet to document
     *
     * @param string $name
     * @return array
     * @since 1.0.0
     */
    public function addSnippetToDocument(int $snippetId, int $documentId, string $language = '')
    {
        $request = [
            "jsonrpc" => "2.0",
            "method" => "Otys.Services.Sdv2StandardDocumentSnippetService.addRecursive",
            "params" => [
                $this->otysApi->session,
                $snippetId,
                $documentId
            ],
            "id" => 1
        ];

        $response = $this->post($request, false, $language);

        return $response;
    }

    /**
     * Create a question for a snippet
     *
     * @param string $name
     * @param string $code
     * @param integer $snippetId
     * @param string $question
     * @param string $type
     * @return array
     */
    public function createQuestion(int $snippetId, string $name, string $code, string $question = '', $type = 'textField', string $language = ''): int
    {
        $existsRequest = [
            "jsonrpc" => "2.0",
            "method" => "Otys.Services.Sdv2QuestionService.getList",
            "params" => [
                $this->otysApi->session,
                [
                    "condition" => [
                        "type" => "COND",
                        "field" => "snippetId",
                        "op" => "EQ",
                        "param" => $snippetId
                    ],
                    "sort" => [
                        "rank" => "ASC"
                    ],
                    "limit" => 200
                ]
            ],
            "id" => 40
        ];

        $existsResponse = $this->post($existsRequest, false, $language);

        // Check if a question with the current code already exists. If so return the uid of the existing question
        if (!empty($existsResponse) && is_array($existsResponse)) {
            foreach ($existsResponse as $snippet) {
                if ($snippet['code'] === $code) {
                    return $snippet['uid'];
                }
            }
        }

        // If question does not yet exist, create a new question
        $request = [
            "jsonrpc" => "2.0",
            "method" => "Otys.Services.Sdv2QuestionService.add",
            "params" => [
                $this->otysApi->session,
                [
                    "name" => "{$name}",
                    "code" => "{$code}",
                    "snippetId" => $snippetId,
                    "question" => ($question !== '' ? $question : $code),
                    "type" => "{$type}"
                ]
            ],
            "id" => 1
        ];

        if (!empty($snippetUid = $this->post($request)) && is_int($snippetUid)) {
            return (int) $this->post($request);
        }

        return 0;
    }

    /**
     * Get list of documents
     *
     * @return array
     * @since 1.0.0
     */
    public function getList(): array
    {
        $response = [];

        $request = [
            "jsonrpc" => "2.0",
            "method" => "Otys.Services.Sdv2StandardDocumentService.getListEx",
            "params" => [
                $this->otysApi->session,
                [
                    "what" => [
                        "uid" => 1,
                        "name" => 1,
                        "isGlobal" => 1,
                        "entryDateTime" => 1,
                        "addedByUserId" => 1,
                        "addedByUser" => 1,
                        "modifiedDateTime" => 1,
                        "modifiedByUserId" => 1,
                        "modifiedByUser" => 1,
                        "category" => 1,
                        "categoryId" => 1,
                        "defaultOutput" => 1
                    ],
                    "limit" => 500,
                    "offset" => 0,
                    "getTotalCount" => true,
                    "sort" => [
                        "name" => "ASC"
                    ],
                    "condition" => [
                        "type" => "AND",
                        "items" => [
                            [
                                "type" => "COND",
                                "field" => "defaultOutput",
                                "op" => "EQ",
                                "param" => "Email"
                            ]
                        ]
                    ]
                ]
            ],
            "id" => 1
        ];

        if (!empty($response = $this->post($request))) {
            return $response['listOutput'];
        }

        return [];
    }

    /**
     * Get translation of document
     *
     * @param integer $uid
     * @return array
     */
    public function getDocumentTranslationUid(int $uid, $languageCode): int
    {
        $request = [
            "jsonrpc" => "2.0",
            "method" => "Otys.Services.Sdv2StandardDocumentService.fetchLanguageVersionUid",
            "params" => [
                $this->otysApi->session,
                $uid,
                "{$languageCode}"
            ],
            "id" => 1
        ];

        $response = $this->post($request, false, false);

        if (is_int($response) || is_string($response)) {
            return intval($response);
        }

        return 0;
    }
}
