<?php

namespace Otys\OtysPlugin\Includes\OtysApi;

class Users extends OtysApiBase
{
    /**
     * getPersonalInformation of a user
     *
     * @param  mixed $userId
     * @return array Returns information of the user if a correct id is specified. If a wrong id is
     * specified the list of settings will return but values of the settings will all be "null"
     * @since 1.0.0
     */
    public function get(int $userId = 0): array
    {
        // Init result
        $result = [];

        /**
         * Since we need the client code we will get the client code from the
         * user which is used to do API calls. We also need the client id to
         * retrieve setting values below
         */
        $clientCode = $this->otysApi->getInfo('clientCode');
        $clientId = $this->otysApi->getInfo('clientId');

        // Create a list which settings to get
        $settingsList = [
            "DB66",
            "DB116",
            "DB123",
            "DB122",
            "DB121",
            "DB128",
            "DB119",
            "DB118",
            "DB120",
            "DB63",
            "DB127",
            "DB124",
            "DB126",
            "DB125",
            "DB115",
            "DB65",
            "DB64",
            "DB117",
            "DB67",
            "DB68"
        ];

        /**
         * Based on the setting list create the item list part
         * of the request which we will use in the OWS request
         */
        $settingRequestItems = [];

        foreach ($settingsList as $setting) {
            $settingRequestItems[] = [
                "type" => "OR",
                "items" => [
                    [
                        "type" => "COND",
                        "field" => "settingId",
                        "op" => "EQ",
                        "param" => "{$setting}"
                    ]
                ]
            ];
        }

        /**
         * Create a ows request. We use CsmService.getListEx to retrieve
         * the settings that we want. This request will contain information
         * about the setting (see the what property).
         *
         * After that request we need to seperataly get the values of those
         * settings. To do this we use the CsmService.getValue, this returns
         * values with the uid of the settings
         *
         * Last but not least we get the photo of the user which we will use
         * UserService.getDetail for which returns some details of the user
         * but since the details of a user are the same fields as settings we
         * will only be getting the photo of the user using this request
         */
        $owsRequest = [
            "jsonrpc" => "2.0",
            "method" => "Otys.Services.MultiService.execute",
            "params" => [
                [
                    [
                        "method" => "Otys.Services.CsmService.getListEx",
                        "args" => [
                            $this->otysApi->session,
                            [
                                "what" => [
                                    "uid" => 1,
                                    "name" => 1,
                                    "defaultName" => 1,
                                    "dataType" => 1,
                                    "useType" => 1,
                                    "logicalType" => 1,
                                    "code" => 1,
                                    "accessLevel" => 1,
                                    "description" => 1,
                                    "Categories" => [
                                        "uid" => 1
                                    ],
                                    "hasDependency" => 1,
                                    "Dependencies" => 1
                                ],
                                "condition" => [
                                    "type" => "AND",
                                    "invert" => false,
                                    "items" => [
                                        [
                                            "type" => "COND",
                                            "field" => "useType",
                                            "op" => "EQ",
                                            "param" => [
                                                "user",
                                                "client-user"
                                            ]
                                        ],
                                        [
                                            "type" => "OR",
                                            "invert" => false,
                                            "items" => $settingRequestItems
                                        ]
                                    ]
                                ],
                                "getTotalCount" => 1,
                                "excludeLimitCheck" => 1,
                                "limit" => 500,
                                "targetUserId" => $userId
                            ]
                        ]
                    ],
                    [
                        "method" => "Otys.Services.CsmService.getValue",
                        "args" => [
                            $this->otysApi->session,
                            $settingsList,
                            $clientId,
                            $userId, // target user
                            0
                        ]
                    ],
                    [
                        "method" => "Otys.Services.UserService.getDetail",
                        "args" => [
                            $this->otysApi->session,
                            $userId,
                            [
                                "Person" => [
                                    "photoFileName" => 1
                                ]
                            ]
                        ]
                    ]
                ]
            ],
            "id" => 1
        ];

        $owsCall = $this->post($owsRequest);
        $owsResult = $owsCall;

        // Check if the OWS request was succesfull
        if (!empty($owsResult)) {
            /**
             * Loop through all current settings create a new array based on the
             * go UID name. This will make it possible to retrieve the information
             * more user friendly and easy to find the information back in go. Also we will assign
             * the current value of the setting to the new key so the information is combined.
             * This needs to happen since it's not possible in OWS to retrieve both the value
             * and the setting name within one call.
             */
            foreach ($owsResult[0]['data']['listOutput'] as $key => $setting) {
                /**
                 * Check if the code key has a value assigned. If the code has no value
                 * the setting won't hold relevant information for this function so we
                 * will exclude it from the result
                 */
                if (isset($setting['code']) && $setting['code'] !== null) {
                    $uid = $setting['uid'];
                    $result['settings'][$setting['uid']] = $setting;
                    $result['settings'][$setting['uid']]['value'] = $owsResult[1]['data'][$uid]['value'];
                    $result['settings'][$setting['uid']]['translated'] = $owsResult[1]['data'][$uid]['translated'];
                }
            }

            // Assign consultant photo to result
            $result['photo'] = ($owsResult[2]['data']) ? $owsResult[2]['data']['Person']['photoFileName'] : null;
            $result['photo_url'] = $result['photo'] !== null ? "https://www.yourit.nl/{$clientCode}/_images_upload/" . $result['photo'] : [];
        }

        return $result;
    }

    public function createRequest(string $request): array
    {
        $request = str_replace('SESSION_ID_REPLACE', $this->otysApi->session, $request);

        $request = json_decode($request, true);


        return $request;
    }
}
