<?php

namespace Otys\OtysPlugin\Models;

use Otys\OtysPlugin\Includes\Core\Cache;
use Otys\OtysPlugin\Includes\OtysApi\OtysApi;

class VacanciesModel extends BaseModel
{

    /**
     * Stores vacancies result from cache or ows
     *
     * @var mixed
     *
     * @since 1.0.0
     */
    protected $vacancies;

    /**
     * Stores list of selected match criteria
     *
     * @var mixed
     *
     * @since 1.0.0
     */
    protected $selectedMatchCriteria;

    /**
     * Get Matchcriteria list slug based
     *
     * @return array
     * @param array $criteria List of criteria to display
     *
     * @since 1.0.0
     */
    public function getMatchCriteriaSlugs($matchCriteriaToShow = null)
    {        
        /**
         * When we have no cache
         */
        $otysApi = new OtysApi();
        
        $matchCriteriaList = $otysApi->matchCriteria->getListSlugs($matchCriteriaToShow);

        return $matchCriteriaList;
    }

    /**
     * Get selected match criteria array to be used in API Call
     *
     * @return array
     *
     * @since 1.0.0
     */
    public function getSelectedMatchCriteria($args = []): array
    {
        $matchCriteriaSlugs = $this->getMatchCriteriaSlugs();
        $get_parameters = $this->getQueryParameters($args);

        // Generate a selected match criteria list
        $selectedMatchCriteria = [];

        // Loop through match criteria slugged and save the ones that are set as $_GET parameter
        foreach ($matchCriteriaSlugs as $criteria_slug => $criteria) {
            if (isset($get_parameters[$criteria_slug])) {
                $selectedMatchCriteria[$criteria['id']]['options']['required'] = true;
                $selectedMatchCriteria[$criteria['id']]['value']['required'] = [];

                // Get options seperate options with plus sign
                $matchCriteriaUrlOptions = explode("+", urlencode($get_parameters[$criteria_slug]));

                // Loop through all options and check if the option is within the url
                foreach ($matchCriteriaUrlOptions as $selectedOption) {
                    if (isset($criteria['options'][$selectedOption]) && array_key_exists('id', $criteria['options'][$selectedOption])) {
                        $selectedMatchCriteria[$criteria['id']]['value']['required'][] = $criteria['options'][$selectedOption]['id'];
                    }
                }
            }
        }

        return $selectedMatchCriteria;
    }

    /**
     * Get matchcriteria list which will be used in the front end
     * The matchcriteria list will contain which options / criteria are active & frequency numbers
     *
     * @return array
     *
     * @since 1.0.0
     */
    public function getMatchCriteriaList($selectedCriteria = []): array
    {
        // Get GET paramaters
        $get_parameters = $this->getQueryParameters($selectedCriteria);

        // Get the list of criteria we want to show
        $matchCriteriaToShow = get_option('otys_option_vacancies_filters_match_criteria');

        $matchCriteriaSlugs = $this->getMatchCriteriaSlugs($matchCriteriaToShow);

        // Generate new Matchcriteria list so we can edit the list
        $matchCriteriaResult = $matchCriteriaSlugs;

        // Validate query parameters and perform action accodingly
        foreach ($get_parameters as $get_key => $get_value) {
            if (empty($get_value)) {
                // If parameter if empty remove the parameter
                unset($get_parameters[$get_key]);
            }
        }

        // Loop through all matchcriteria
        foreach ($matchCriteriaSlugs as $criteria_slug => $criteria) {
            // Generate new GET URL

            if (isset($get_parameters[$criteria_slug])) {
                // Get options seperate options with plus sign
                $get_options = explode("+", urlencode($get_parameters[$criteria_slug]));
                $get_options_count = count($get_options);
            } else {
                $get_options = false;
            }

            // Set active status current criteria
            if (array_key_exists($criteria_slug, $get_parameters)) {
               $matchCriteriaResult[$criteria_slug]['active'] = true;
            }

            // SET URL By Looping through all options of the current matchcriteria and deciding what the url must be
            foreach ($criteria['options'] as $criteria_option_slug => $criteria_option) {
                // Set new_get variable to generate url for this option
                $new_url = $get_parameters;

                // Set initial value for url, this should never result in nothing, but just to make sure to return something
                $url = '';

                if (!isset($get_parameters)) {
                    // When there is no get set we can expect the url to be as simple as ?criterianame=option
                    $url = '?' . $criteria_slug . '=' . $criteria_option_slug;
                } elseif ($get_options !== false && array_key_exists($criteria_slug, $get_parameters)) {
                    // There are options set for the current matchcriteria

                    // Set a new options variable which we will use to generate a new options string
                    $newoptions = $get_options;

                    // Check if the current option is within the GET
                    if (($option_key = array_search($criteria_option_slug, $newoptions)) !== false) {
                        // Since the option is currently selected we want to deselect the option and therefore we remove the option from the optionlist
                        unset($newoptions[$option_key]);

                        // Since the current option is selected we also want to add the active status to the current option in the match criteria list
                        $matchCriteriaResult[$criteria_slug]['options'][$criteria_option_slug]['active'] = true;
                    } else {
                        // Since the curent option is not currently selected we want to add the option to the option list
                        $newoptions[] = $criteria_option_slug;
                    }

                    // Decide wheter to keep the current matchcriteria in the new url
                    if (empty($newoptions)) {
                        // If the new option list will be empty remove the entire matchcriteria
                        unset($new_url[$criteria_slug]);
                    } else {
                        // If there are still options create the new options list
                        $new_url[$criteria_slug] = implode(" ", $newoptions);
                    }
                } else {
                    // The current matchcriteria is not yet in the url
                    $new_url[$criteria_slug] = $criteria_option_slug;
                }

                // Place search parameter at the end of the url
                if (!empty($new_url['s'])) {
                    $search_value = $new_url['s'];
                    unset($new_url['s']);
                    $new_url['s'] = $search_value;
                }

                // Build new url get parameters
                $url = http_build_query($new_url);

                // Assign generated url to the current option
                $matchCriteriaResult[$criteria_slug]['options'][$criteria_option_slug]['url'] = $url;
            }
        }

        return $matchCriteriaResult;
    }

    /**
     * Get a list of allowed query parameters
     *
     * @return array
     *
     * @since 1.0.0
     */
    public function getQueryParameters($selectedCriteria = []): array
    {
        // Skip get parameters and return empty array
        if ((!isset($_GET) || empty($_GET)) && empty($selectedCriteria)) {
            return [];
        }

        $matchCriteriaSlugged = $this->getMatchCriteriaSlugs();
        $return = [];
        
        foreach ($matchCriteriaSlugged as $matchCriteriaSlug => $matchCriteria) {
            // Check if current match criteria is in selected criteria
            if (
                array_key_exists($matchCriteria['id'], $selectedCriteria) ||
                array_key_exists($matchCriteriaSlug, $selectedCriteria)
            ) {
                $selectedCriteriaFiltered = [];

                foreach ($matchCriteria['options'] as $optionSlug => $option) {
                    if (
                        (isset($selectedCriteria[$matchCriteria['id']]) && 
                        in_array($option['id'], $selectedCriteria[$matchCriteria['id']]))
                        ||
                        (isset($selectedCriteria[$matchCriteriaSlug]) &&
                        in_array($optionSlug, $selectedCriteria[$matchCriteriaSlug]))
                    ) {
                        $selectedCriteriaFiltered[] = $optionSlug;
                    }
                }

                $return[$matchCriteriaSlug] = implode(" ", $selectedCriteriaFiltered);

                continue;
            }

            if (isset($_GET[$matchCriteriaSlug])) {
                $getParameterOptions = explode("+", urlencode(esc_attr($_GET[$matchCriteriaSlug])));

                // Check if the options given are allowed
                foreach ($getParameterOptions as $parameterOptionKey => $parameterOption) {
                    if (!array_key_exists($parameterOption, $matchCriteriaSlugged[$matchCriteriaSlug]['options'])) {
                        unset($getParameterOptions[$parameterOptionKey]);
                    }
                }

                $return[$matchCriteriaSlug] = implode(" ", $getParameterOptions);
            }
        }

        return $return;
    }

    /**
     * Get selected criteria list with links
     *
     * @return void
     *
     * @since 1.0.0
     */
    public function getSelectedCriteria($selectedCriteria = []): array
    {
        if ($this->selectedMatchCriteria !== null) {
            return $this->selectedMatchCriteria;
        }

        $query_parameters = $this->getQueryParameters($selectedCriteria);
        $match_criteria_slugs = $this->getMatchCriteriaSlugs();

        // List of selected parameters that we will build and later return
        $selected_parameters = [];

        // remove page from query parameters since we don't want to stay on the same page when removing a parameter
        unset($query_parameters['page-number']);

        // Loop through all query parameters and build the selected criteria list
        foreach ($query_parameters as $parameter_name => $parameter_value) {
            $is_criteria = array_key_exists($parameter_name, $match_criteria_slugs); // contains if current parameter is a matchcriteria

            // If parameter is not a matchcriteria or search keyword skip the current parameter
            if (!$is_criteria && $parameter_name !== 's') {
                continue;
            }

            $new_query_parameters = $query_parameters;

            // Remove the current parameter from the url to generate the new url without the parameter
            unset($new_query_parameters[$parameter_name]);

            // Set new url
            $selected_parameters[$parameter_name]['url'] = http_build_query($new_query_parameters);

            if ($is_criteria) {
                // If is criteria return create names belonging to the slug
                $slug_values = explode(" ", $parameter_value);
                $name_values = [];

                foreach ($slug_values as $option_slug) {
                    $name_values[] = $match_criteria_slugs[$parameter_name]['options'][$option_slug]['name'];
                }

                $selected_parameters[$parameter_name]['name'] = $name_values;
                $selected_parameters[$parameter_name]['type'] = 'match_criteria';
            } elseif ($parameter_name === 's') {
                // If is search parameter return search value
                $selected_parameters[$parameter_name]['name'][] = $parameter_value;
                $selected_parameters[$parameter_name]['type'] = 'search';
            }
        }

        $this->selectedMatchCriteria = $selected_parameters;

        return $selected_parameters;
    }
}